@@ -3,22 +3,13 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2014 Jean-François Moine
+ * Copyright (C) 1998-2017 Jean-François Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
 #include <stdarg.h>
@@ -26,12 +17,9 @@
 #include <string.h>
 #include <ctype.h>
 
-#ifdef WIN32
-#define snprintf _snprintf
-#endif
-
-#include "abc2ps.h" 
+#include "abcm2ps.h" 
 
+#define PPI_96_72 0.75		// convert page format to 72 PPI
 #define BUFFLN	80		/* max number of lines in output buffer */
 
 static int ln_num;		/* number of lines in buffer */
@@ -43,7 +31,8 @@ static signed char ln_font[BUFFLN];	/* font of buffered lines */
 static float cur_lmarg = 0;	/* current left margin */
 static float min_lmarg, max_rmarg;	/* margins for -E/-g */
 static float cur_scale = 1.0;	/* current scale */
-static float maxy;		/* remaining vertical space in page */
+static float maxy;		/* usable vertical space in page */
+static float remy;		/* remaining vertical space in page */
 static float bposy;		/* current position in buffered data */
 static int nepsf;		/* counter for -E/-g output files */
 static int nbpages;		/* number of pages in the output file */
@@ -70,7 +59,7 @@ static void cutext(char *fid)
 }
 
 /* -- open the output file -- */
-static void open_fout(void)
+void open_fout(void)
 {
 	int i;
 	char fnm[FILENAME_MAX];
@@ -78,29 +67,20 @@ static void open_fout(void)
 	strcpy(fnm, outfn);
 	i = strlen(fnm) - 1;
 	if (i < 0) {
-		strcpy(fnm, svg ? "Out.xhtml" : OUTPUTFILE);
-#if 1
+		strcpy(fnm, svg || epsf > 1 ? "Out.xhtml" : OUTPUTFILE);
 	} else if (i != 0 || fnm[0] != '-') {
-#else
-	} else if (i == 0 && fnm[0] == '-') {
-		if (svg == 1) {
-			error(1, 0, "Cannot use stdout with '-v' - abort");
-			exit(EXIT_FAILURE);
-		}
-	} else {
-#endif
-		if (fnm[i] == '=') {
+		if (fnm[i] == '=' && in_fname) {
 			char *p;
 
-			if ((p = strrchr(in_fname, DIRSEP)) == 0)
+			if ((p = strrchr(in_fname, DIRSEP)) == NULL)
 				p = in_fname;
 			else
 				p++;
-/*fixme: should check if there is a DIRSEP at the end of fnm*/
 			strcpy(&fnm[i], p);
-			strext(fnm, svg ? "xhtml" : "ps");
+			strext(fnm, svg || epsf > 1 ? "xhtml" : "ps");
 		} else if (fnm[i] == DIRSEP) {
-			strcpy(&fnm[i + 1], OUTPUTFILE);
+			strcpy(&fnm[i + 1],
+				svg || epsf > 1 ? "Out.xhtml" : OUTPUTFILE);
 		}
 #if 0
 /*fixme: fnm may be a directory*/
@@ -153,23 +133,23 @@ static void init_ps(char *str)
 {
 	time_t ltime;
 	unsigned i;
-	char version[32];
+	char version[] = "/creator [(abcm2ps) " VERSION "] def";
 
 	if (epsf) {
 		cur_lmarg = min_lmarg - 10;
 		fprintf(fout, "%%!PS-Adobe-2.0 EPSF-2.0\n"
 			"%%%%BoundingBox: 0 0 %.0f %.0f\n",
-			(p_fmt->landscape ? p_fmt->pageheight : p_fmt->pagewidth)
-				- cur_lmarg - max_rmarg + 10,
-			-bposy);
+			((p_fmt->landscape ? p_fmt->pageheight : p_fmt->pagewidth)
+				- cur_lmarg - max_rmarg + 10) * PPI_96_72,
+			-bposy * PPI_96_72);
 		marg_init();
 	} else {
 		if (!fout)
 			open_fout();
 		fprintf(fout, "%%!PS-Adobe-2.0\n");
 		fprintf(fout, "%%%%BoundingBox: 0 0 %.0f %.0f\n",
-			p_fmt->pagewidth,
-			p_fmt->pageheight);
+			p_fmt->pagewidth * PPI_96_72,
+			p_fmt->pageheight * PPI_96_72);
 	}
 	fprintf(fout, "%%%%Title: %s\n", str);
 	time(&ltime);
@@ -208,7 +188,6 @@ static void init_ps(char *str)
 	fprintf(fout, "\n\n");
 	if (epsf)
 		fprintf(fout, "save\n");
-	strcpy(version, "/creator [(abcm2ps) " VERSION "] def");
 	for (i = 0; i < strlen(version); i++) {
 		if (version[i] == '.')
 			version[i] = ' ';
@@ -226,7 +205,6 @@ static void init_ps(char *str)
 		"/SLW/setlinewidth load def\n"
 		"/defl 0 def\n"	/* decoration flags - see deco.c for values */
 		"/dlw{0.7 SLW}!\n"
-
 		"%s\n", version);
 	define_symbols();
 	output = fprintf;
@@ -235,7 +213,8 @@ static void init_ps(char *str)
 	if (!epsf)
 		fprintf(fout, "/setpagedevice where{pop\n"
 			"	<</PageSize[%.0f %.0f]>>setpagedevice}if\n",
-				p_fmt->pagewidth, p_fmt->pageheight);
+				p_fmt->pagewidth * PPI_96_72,
+				p_fmt->pageheight * PPI_96_72);
 	fprintf(fout, "%%%%EndSetup\n");
 	file_initialized = 1;
 }
@@ -246,8 +225,8 @@ static void init_svg(char *str)
 	cur_lmarg = min_lmarg - 10;
 	output = svg_output;
 #if 1 //fixme:test
-	if (file_initialized)
-		fprintf(stderr, "??? init_svg: file_initialized");
+	if (file_initialized > 0)
+		fprintf(stderr, "??? init_svg: file_initialized\n");
 #endif
 	define_svg_symbols(str, nepsf,
 		(p_fmt->landscape ? p_fmt->pageheight : p_fmt->pagewidth)
@@ -297,17 +276,25 @@ void close_output_file(void)
 	if (tunenum == 0)
 		error(0, NULL, "No tunes written to output file");
 	close_page();
-	if (!svg) {
-		fprintf(fout, "%%%%Trailer\n"
-			"%%%%Pages: %d\n"
-			"%%EOF\n", nbpages);
+	switch (svg) {
+	case 0:				/* PS */
+		if (epsf == 0)
+			fprintf(fout, "%%%%Trailer\n"
+				"%%%%Pages: %d\n"
+				"%%EOF\n", nbpages);
 		close_fout();
-	} else if (svg == 2) {
+		break;
+	case 2:				/* -X */
 		fputs("</body>\n"
 			"</html>\n", fout);
+	case 3:				/* -z */
 		close_fout();
-	}				/* else (svg == 1)
-					 * 'fout' is closed in close_page */
+		break;
+//	default:
+//	case 1:				/* -v */
+//		'fout' is closed in close_page
+	}
+
 	nbpages = tunenum = 0;
 	defl = 0;
 }
@@ -322,23 +309,18 @@ void close_page(void)
 		svg_close();
 		if (svg == 1 && fout != stdout)
 			close_fout();
-		else
-			fputs("</p>\n", fout);
+//		else
+//			fputs("</p>\n", fout);
 	} else {
-#if 1
 		fprintf(fout, "grestore\n"
 				"showpage\n"
 				"%%%%EndPage: %d %d\n",
 				nbpages, nbpages);
-#else
-		fputs("%%PageTrailer\n"
-			"grestore\n"
-			"showpage\n", fout);
-#endif
 	}
 	cur_lmarg = 0;
 	cur_scale = 1.0;
 	outft = -1;
+	use_buffer = 0;
 }
 
 /* -- output a header/footer element -- */
@@ -390,7 +372,7 @@ static void format_hf(char *d, char *p)
 			p++;
 			if (*p < 'A' || *p > 'Z' || !info[*p - 'A'])
 				break;
-			d += sprintf(d, "%s", &info[*p - 'A']->as.text[2]);
+			d += sprintf(d, "%s", &info[*p - 'A']->text[2]);
 			break;
 		case 'P':		/* page number */
 			if (p[1] == '0') {
@@ -405,7 +387,7 @@ static void format_hf(char *d, char *p)
 			d += sprintf(d, "%d", pagenum);
 			break;
 		case 'T':		/* tune title */
-			q = &info['T' - 'A']->as.text[2];
+			q = &info['T' - 'A']->text[2];
 			tex_str(q);
 			d += sprintf(d, "%s", tex_buf);
 			break;
@@ -425,11 +407,12 @@ static float headfooter(int header,
 			float pwidth,
 			float pheight)
 {
-	char tmp[TEX_BUF_SZ], str[TEX_BUF_SZ + 1024];
-	char *p, *q, *r, *mbf_sav;
+	char tmp[2048], str[TEX_BUF_SZ + 512];
+	char *p, *q, *r, *outbuf_sav, *mbf_sav;
 	float size, y, wsize;
 	struct FONTSPEC *f, f_sav;
-	int cft_sav, dft_sav;
+
+	int cft_sav, dft_sav, outbufsz_sav;
 
 	if (header) {
 		p = cfmt.header;
@@ -463,6 +446,10 @@ static float headfooter(int header,
 		*r = '\0';
 	}
 	mbf_sav = mbf;
+	outbuf_sav = outbuf;
+	outbufsz_sav = outbufsz;
+	outbuf = tmp;
+	outbufsz = sizeof tmp;
 	for (;;) {
 		tex_str(p);
 		strcpy(tmp, tex_buf);
@@ -475,13 +462,13 @@ static float headfooter(int header,
 				*q = '\0';
 				output(fout, "%.1f %.1f M ",
 					p_fmt->leftmargin, y);
+				mbf = tmp;
 				str_out(p, A_LEFT);
 				a2b("\n");
-				mbf = mbf_sav;
 				if (svg)
-					svg_write(mbf, strlen(mbf));
+					svg_write(tmp, strlen(tmp));
 				else
-					fputs(mbf, fout);
+					fputs(tmp, fout);
 			}
 			p = q + 1;
 		}
@@ -492,13 +479,13 @@ static float headfooter(int header,
 		if (q != p) {
 			output(fout, "%.1f %.1f M ",
 				pwidth * 0.5, y);
+			mbf = tmp;
 			str_out(p, A_CENTER);
 			a2b("\n");
-			mbf = mbf_sav;
 			if (svg)
-				svg_write(mbf, strlen(mbf));
+				svg_write(tmp, strlen(tmp));
 			else
-				fputs(mbf, fout);
+				fputs(tmp, fout);
 		}
 
 		/* right side */
@@ -507,13 +494,13 @@ static float headfooter(int header,
 			if (*p != '\0') {
 				output(fout, "%.1f %.1f M ",
 					pwidth - p_fmt->rightmargin, y);
+				mbf = tmp;
 				str_out(p, A_RIGHT);
 				a2b("\n");
-				mbf = mbf_sav;
 				if (svg)
-					svg_write(mbf, strlen(mbf));
+					svg_write(tmp, strlen(tmp));
 				else
-					fputs(mbf, fout);
+					fputs(tmp, fout);
 			}
 		}
 		if (!r)
@@ -524,8 +511,10 @@ static float headfooter(int header,
 		y -= size;
 	}
 
-	/* restore the fonts */
-	*mbf_sav = '\0';
+	/* restore the buffer and fonts */
+	outbuf = outbuf_sav;
+	outbufsz = outbufsz_sav;
+	mbf = mbf_sav;
 	memcpy(&cfmt.font_tb[0], &f_sav, sizeof cfmt.font_tb[0]);
 	set_str_font(cft_sav, dft_sav);
 	return wsize;
@@ -541,12 +530,13 @@ static void init_page(void)
 
 	nbpages++;
 	if (svg) {
-		if (!file_initialized) {
+		if (file_initialized <= 0) {
 			if (!fout)
 				open_fout();
 			define_svg_symbols(in_fname, nbpages,
 				cfmt.landscape ? p_fmt->pageheight : p_fmt->pagewidth,
 				cfmt.landscape ? p_fmt->pagewidth : p_fmt->pageheight);
+			user_ps_write();
 			file_initialized = 1;
 			output = svg_output;
 		} else {
@@ -554,8 +544,7 @@ static void init_page(void)
 				cfmt.landscape ? p_fmt->pageheight : p_fmt->pagewidth,
 				cfmt.landscape ? p_fmt->pagewidth : p_fmt->pageheight);
 		}
-		user_ps_write();
-	} else if (!file_initialized) {
+	} else if (file_initialized <= 0) {
 		init_ps(in_fname);
 	}
 	in_page = 1;
@@ -569,13 +558,13 @@ static void init_page(void)
 		pwidth = cfmt.pageheight;
 		if (!svg)
 			fprintf(fout, "%%%%PageOrientation: Landscape\n"
-				"gsave 90 rotate 0 %.1f T\n",
+				"gsave 0.75 dup scale 90 rotate 0 %.1f T\n",
 				-cfmt.topmargin);
 	} else {
 		pheight = cfmt.pageheight;
 		pwidth = p_fmt->pagewidth;
 		if (!svg)
-			fprintf(fout, "gsave 0 %.1f T\n",
+			fprintf(fout, "gsave 0.75 dup scale 0 %.1f T\n",
 				pheight - cfmt.topmargin);
 	}
 	if (svg)
@@ -583,10 +572,13 @@ static void init_page(void)
 	else
 		output(fout,
 			"%% --- width %.1f\n",		/* for index */
-			((cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
-			 - cfmt.leftmargin - cfmt.rightmargin) / cfmt.scale);
+			(pwidth - cfmt.leftmargin - cfmt.rightmargin) /
+					cfmt.scale);
+	if (cfmt.gutter != 0)
+		output(fout, "%.1f 0 T\n", (pagenum & 1) ?
+					cfmt.gutter : -cfmt.gutter);
 
-	maxy = pheight - cfmt.topmargin - cfmt.botmargin;
+	remy = maxy = pheight - cfmt.topmargin - cfmt.botmargin;
 
 	/* output the header and footer */
 	if (!cfmt.header) {
@@ -607,11 +599,11 @@ static void init_page(void)
 		dy = headfooter(1, pwidth, pheight);
 		if (dy != 0) {
 			output(fout, "0 %.1f T\n", -dy);
-			maxy -= dy;
+			remy -= dy;
 		}
 	}
 	if (cfmt.footer)
-		maxy -= headfooter(0, pwidth, pheight);
+		remy -= headfooter(0, pwidth, pheight);
 	pagenum++;
 	outft = -1;
 }
@@ -635,7 +627,7 @@ static void epsf_title(char *p, int sz)
 {
 	unsigned char c;
 
-	snprintf(p, sz, "%.72s (%.4s)", in_fname, &info['X' - 'A']->as.text[2]);
+	snprintf(p, sz, "%.72s (%.4s)", in_fname, &info['X' - 'A']->text[2]);
 	while ((c = (unsigned char) *p) != '\0') {
 		if (c >= 0x80) {
 			if ((c & 0xf8) == 0xf0) {
@@ -670,61 +662,60 @@ void write_eps(void)
 
 	p_fmt = &cfmt;				/* tune format */
 
-	strcpy(outfnam, outfn);
-	if (outfnam[0] == '\0')
-		strcpy(outfnam, OUTPUTFILE);
-	cutext(outfnam);
-	i = strlen(outfnam) - 1;
-	if (i == 0 && outfnam[0] == '-') {
-		if (epsf == 1) {
-			error(1, NULL, "Cannot use stdout with '-E' - abort");
-			exit(EXIT_FAILURE);
-		}
-		fout = stdout;
-	} else {
-		if (outfnam[i] == '=') {
-			p = &info['T' - 'A']->as.text[2];
-			while (isspace((unsigned char) *p))
-				p++;
-			strncpy(&outfnam[i], p, sizeof outfnam - i - 4);
-			outfnam[sizeof outfnam - 5] = '\0';
-			epsf_fn_adj(&outfnam[i]);
+	if (epsf != 3) {			/* if not -z */
+		strcpy(outfnam, outfn);
+		if (outfnam[0] == '\0')
+			strcpy(outfnam, OUTPUTFILE);
+		cutext(outfnam);
+		i = strlen(outfnam) - 1;
+		if (i == 0 && outfnam[0] == '-') {
+			if (epsf == 1) {
+				error(1, NULL, "Cannot use stdout with '-E' - abort");
+				exit(EXIT_FAILURE);
+			}
+			fout = stdout;
 		} else {
-			if (i >= sizeof outfnam - 4 - 3)
-				i = sizeof outfnam - 4 - 3;
-			sprintf(&outfnam[i + 1], "%03d", ++nepsf);
-		}
-		strcat(outfnam, epsf == 1 ? ".eps" : ".svg");
-		if ((fout = fopen(outfnam, "w")) == NULL) {
-			error(1, NULL, "Cannot open output file %s - abort",
-					outfnam);
-			exit(EXIT_FAILURE);
+			if (outfnam[i] == '=') {
+				p = &info['T' - 'A']->text[2];
+				while (isspace((unsigned char) *p))
+					p++;
+				strncpy(&outfnam[i], p, sizeof outfnam - i - 4);
+				outfnam[sizeof outfnam - 5] = '\0';
+				epsf_fn_adj(&outfnam[i]);
+			} else {
+				if (i >= sizeof outfnam - 4 - 3)
+					i = sizeof outfnam - 4 - 3;
+				sprintf(&outfnam[i + 1], "%03d", ++nepsf);
+			}
+			strcat(outfnam, epsf == 1 ? ".eps" : ".svg");
+			if ((fout = fopen(outfnam, "w")) == NULL) {
+				error(1, NULL, "Cannot open output file %s - abort",
+						outfnam);
+				exit(EXIT_FAILURE);
+			}
 		}
 	}
 	epsf_title(title, sizeof title);
 	if (epsf == 1) {
 		init_ps(title);
-		fprintf(fout, "0 %.1f T\n", -bposy);
+		fprintf(fout, "0.75 dup scale 0 %.1f T\n", -bposy);
 		write_buffer();
 		fprintf(fout, "showpage\nrestore\n");
 	} else {
+		if (epsf == 3 && file_initialized == 0)
+			fputs("<br/>\n", fout);	/* new image in the same flow */
 		init_svg(title);
 		write_buffer();
 		svg_close();
 	}
-	close_fout();
+	if (epsf != 3)
+		close_fout();
+	else
+		file_initialized = 0;
 	cur_lmarg = 0;
 	cur_scale = 1.0;
 }
 
-/* -- start a new page -- */
-/* epsf is always null */
-static void newpage(void)
-{
-	close_page();
-	init_page();
-}
-
 /*  subroutines to handle output buffer  */
 
 /* -- update the output buffer pointer -- */
@@ -740,7 +731,7 @@ void a2b(char *fmt, ...)
 		}
 		error(0, NULL, "Possible buffer overflow");
 		write_buffer();
-		use_buffer = 0;
+//		use_buffer = 0;
 	}
 	va_start(args, fmt);
 	mbf += vsnprintf(mbf, outbuf + outbufsz - mbf, fmt, args);
@@ -750,6 +741,8 @@ void a2b(char *fmt, ...)
 /* -- translate down by 'h' scaled points in output buffer -- */
 void bskip(float h)
 {
+	if (h == 0)
+		return;
 	bposy -= h * cfmt.scale;
 	a2b("0 %.2f T\n", -h);
 }
@@ -760,8 +753,8 @@ void init_outbuf(int kbsz)
 	if (outbuf)
 		free(outbuf);
 	outbufsz = kbsz * 1024;
-	if (outbufsz < 0x10000)
-		outbufsz = 0x10000;
+//	if (outbufsz < 0x10000)
+//		outbufsz = 0x10000;
 	outbuf = malloc(outbufsz);
 	if (!outbuf) {
 		error(1, NULL, "Out of memory for outbuf - abort");
@@ -802,9 +795,10 @@ void write_buffer(void)
 			}
 		}
 		dp = ln_pos[l] - p1;
-		np = maxy + dp < 0 && !epsf;
+		np = remy + dp < 0 && !epsf;
 		if (np) {
-			newpage();
+			close_page();
+			init_page();
 			if (ln_font[l] >= 0) {
 				struct FONTSPEC *f;
 
@@ -825,10 +819,10 @@ void write_buffer(void)
 		}
 		if (np) {
 			output(fout, "0 %.2f T\n", -cfmt.topspace);
-			maxy -= cfmt.topspace * cfmt.scale;
+			remy -= cfmt.topspace * cfmt.scale;
 		}
 		if (*p_buf != '\001') {
-			if (epsf == 2 || svg)
+			if (epsf > 1 || svg)
 				svg_write(p_buf, ln_buf[l] - p_buf);
 			else
 				fwrite(p_buf, 1, ln_buf[l] - p_buf, fout);
@@ -846,7 +840,7 @@ void write_buffer(void)
 			if ((f = fopen(p, "r")) == NULL) {
 				error(1, NULL, "Cannot open EPS file '%s'", p);
 			} else {
-				if (epsf == 2 || svg) {
+				if (epsf > 1 || svg) {
 					fprintf(fout, "<!--Begin document %s-->\n",
 							p);
 					svg_output(fout, "gsave\n"
@@ -875,27 +869,32 @@ void write_buffer(void)
 			}
 		}
 		p_buf = ln_buf[l];
-		maxy += dp;
+		remy += dp;
 		p1 = ln_pos[l];
 	}
 #if 1 //fixme:test
-	if (*p_buf != '\0')
-		fprintf(stderr, "??? bug - buffer not empty:\n%s\n", p_buf);
+	if (*p_buf != '\0') {
+//		fprintf(stderr, "??? bug - buffer not empty:\n%s\n", p_buf);
+		memcpy(outbuf, p_buf, strlen(p_buf) + 1);
+		mbf = &outbuf[strlen(outbuf)];
+	} else {
+		mbf = outbuf;
+	}
 #endif
 	outft = outft_sav;
 	bposy = 0;
 	ln_num = 0;
-	mbf = outbuf;
+	use_buffer = 0;
 }
 
-/* -- add a block in the output buffer -- */
+/* -- add a block of commmon margins / scale in the output buffer -- */
 void block_put(void)
 {
 	if (mbf == outbuf)
 		return;
 //fixme: should be done sooner and should be adjusted when cfmt change...
-	if (maxy == 0)
-		maxy = (cfmt.landscape ? cfmt.pagewidth : cfmt.pageheight)
+	if (remy == 0)
+		remy = maxy = (cfmt.landscape ? cfmt.pagewidth : cfmt.pageheight)
 			- cfmt.topmargin - cfmt.botmargin;
 	if (ln_num > 0 && mbf == ln_buf[ln_num - 1])
 		return;				/* no data */
@@ -909,10 +908,10 @@ void block_put(void)
 		c = *p;				/* (avoid "buffer not empty") */
 		*p = '\0';
 		write_buffer();
-		multicol_start = maxy + bposy;
+		multicol_start = remy + bposy;
 		*p = c;
 		strcpy(outbuf, p);
-		use_buffer = 0;
+//		use_buffer = 0;
 	}
 	ln_buf[ln_num] = mbf;
 	ln_pos[ln_num] = multicol_start == 0 ? bposy : 1;
@@ -927,26 +926,65 @@ void block_put(void)
 	ln_font[ln_num] = outft;
 	ln_num++;
 
-	if (!use_buffer) {
+	if (!use_buffer)
 		write_buffer();
-		return;
-	}
 }
 
 /* -- handle completed block in buffer -- */
 /* if the added stuff does not fit on current page, write it out
    after page break and change buffer handling mode to pass though */
-void buffer_eob(void)
+void buffer_eob(int eot)
 {
 	block_put();
-	if (maxy + bposy < 0
-	 && !epsf
-	 && multicol_start == 0) {
-		if (in_page)
-			newpage();
-		write_buffer();
-		use_buffer = 0;
+	if (epsf) {
+		if (epsf == 3)
+			write_eps();		/* close the image */
+		return;
 	}
+	if (remy + bposy >= 0
+	 || multicol_start != 0)
+		return;
+
+	// page full
+	if (!in_page) {
+		if ((cfmt.splittune == 2 && !(nbpages & 1))
+		 || (cfmt.splittune == 3 && (nbpages & 1))) { // wrong odd/even page
+			if (remy + maxy + bposy < 0) {	// 2nd overflow
+				init_page();
+				close_page();
+				write_buffer();
+				return;
+			}
+			if (eot)
+				write_buffer();
+			return;
+		}
+	} else {
+		close_page();
+	}
+	if ((cfmt.splittune == 2 && !(nbpages & 1))
+	 || (cfmt.splittune == 3 && (nbpages & 1)))
+		use_buffer = 1;			// if wrong odd/even page
+	else
+		write_buffer();
+#if 0
+--- old
+		if (use_buffer
+		 && !eot
+		 && ((cfmt.splittune == 2 && !(nbpages & 1))
+		   || (cfmt.splittune == 3 && (nbpages & 1)))) {
+			init_page();
+// 8.7.4
+			use_buffer = 1;
+		} else {
+// 8.7.5 - required to avoid buffer overflow
+			write_buffer();
+		}
+//8.7.0
+//		write_buffer();
+//8.6.2
+//		use_buffer = 0;
+#endif
 }
 
 /* -- dump buffer if not enough place for a music line -- */
@@ -956,12 +994,12 @@ void check_buffer(void)
 		error(0, NULL,
 		      "Possibly bad page breaks, outbufsz exceeded");
 		write_buffer();
-		use_buffer = 0;
+//		use_buffer = 0;
 	}
 }
 
 /* -- return the current vertical offset in the page -- */
 float get_bposy(void)
 {
-	return maxy + bposy;
+	return remy + bposy;
 }
