@@ -3,40 +3,33 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 2011-2014 Jean-François Moine (http://moinejf.free.fr)
+ * Copyright (C) 2011-2017 Jean-François Moine (http://moinejf.free.fr)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <regex.h>
 
 #ifdef WIN32
-#define strncasecmp strnicmp
+#define strncasecmp _strnicmp
+#define strdup _strdup
 #endif
 
-#include "front.h"
-#include "slre.h"
+#include "abcm2ps.h"
 
 static unsigned char *dst;
-static int offset, size, keep_comments;
-static void (*include_f)(unsigned char *fn);
+static int offset, size;
 static unsigned char *selection;
 static int latin, skip;
 static char prefix[4] = {'%'};
+static int state;
 
 /*
  * translation table from the ABC draft version 2
@@ -64,7 +57,7 @@ static unsigned char circumflex[] = "AÂEÊIÎOÔUÛaâeêiîoôuûHĤJĴhĥjĵC
 static unsigned char cedilla[] = "CÇcçSŞsşTŢtţRŖLĻGĢrŗlļgģNŅKĶnņkķ";
 static unsigned char umlaut[] = "AÄEËIÏOÖUÜYŸaäeëiïoöuüyÿ";
 static unsigned char tilde[] = "AÃNÑOÕaãnñoõIĨiĩUŨuũ";
-static unsigned char ring[] = "AÅaåUŮuů";
+static unsigned char ring[] = "AÅaåUŮuůeœ";
 static unsigned char macron[] = "AĀDĐEĒHĦIĪOŌTŦUŪaādđeēhħiīoōtŧuū"; /* and stroke! */
 static unsigned char slash[] = "OØoøDĐdđLŁlł";
 static unsigned char ogonek[] = "AĄEĘIĮUŲaąeęiįuų";
@@ -73,7 +66,7 @@ static unsigned char breve[] = "AĂaăEĔeĕGĞgğIĬiĭOŎoŏUŬuŭ";
 static unsigned char hungumlaut[] = "OŐUŰoőuű";
 static unsigned char dot[] = "ZŻzżIİiıCĊcċGĠgġEĖeė";
 /* the items of this table are 4 bytes long */
-static unsigned char ligature[] = "AAÅaaåAEÆaeæccçcCÇDHÐdhðngŋOEŒoeœssßTHÞthþ";
+static unsigned char ligature[] = "AAÅaaåAEÆaeæccçcCÇDHÐdhðngŋOEŒssßTHÞthþ";
 
 /* latin conversion tables - range 0xa0 .. 0xff */
 static unsigned char latin2[] = {
@@ -112,11 +105,11 @@ static void txt_add(unsigned char *s, int sz)
 		return;
 	if (offset + sz > size) {
 		size = (offset + sz + 8191) / 8192 * 8192;
-		if (dst == 0)
+		if (!dst)
 			dst = malloc(size);
 		else
 			dst = realloc(dst, size);
-		if (dst == 0) {
+		if (!dst) {
 			fprintf(stderr, "Out of memory - abort\n");
 			exit(EXIT_FAILURE);
 		}
@@ -127,13 +120,28 @@ static void txt_add(unsigned char *s, int sz)
 
 /* add text to the output buffer translating
  * the escape sequences and the non utf-8 characters */
-static void txt_add_cnv(unsigned char *s, int sz)
+static void txt_add_cnv(unsigned char *s, int sz, int comment)
 {
 	unsigned char *p, c, tmp[4];
+	int in_string = 0;
 
 	p = s;
 	while (sz > 0) {
-		if (*p == '\\') {
+		switch (*p) {
+		case '"':
+			if (comment)
+				in_string = !in_string;
+			break;
+		case '%':
+			if (in_string || !comment)
+				break;
+			while (--p >= s) {	// start of comment
+				if (*p != ' ' && *p != '\t')
+					break;
+			}
+			p++;
+			goto done;
+		case '\\':
 			if (sz >= 4			/* \ooo */
 			 && p[1] >= '0' && p[1] <= '3'
 			 && p[2] >= '0' && p[2] <= '7'
@@ -323,7 +331,7 @@ static void txt_add_cnv(unsigned char *s, int sz)
 			}
 			p++;
 			sz--;
-			if (*p == '\\') {
+			if (sz > 0) {
 				p++;
 				sz--;
 			}
@@ -351,40 +359,45 @@ latin:
 		p++;
 		sz--;
 	}
+done:
 	if (p != s)
 		txt_add(s, (int) (p - s));
 }
 
-static const unsigned char eol_chars[2] = {'\r', '\n'};
-static void eol0(void)
+static void txt_add_eos(char *fname, int linenum)
 {
-	txt_add((unsigned char *) &eol_chars[1], 1);
-}
-static void eol1(void)
-{
-	txt_add((unsigned char *) &eol_chars[0], 1);
-}
-static void eol2(void)
-{
-	txt_add((unsigned char *) &eol_chars[0], 2);
-}
-static void (*eol_tb[3])(void) = {eol0, eol1, eol2};
-static void (*txt_add_eol)(void);
+	static unsigned char eos = '\0';
 
-/* add the line number */
-static void add_lnum(int nline)
-{
-	unsigned char tmp[16];
+	/* special case for continuation lines in ABC version 2.0 */
+	if (parse.abc_vers == (2 << 16)
+	 && offset > 0
+	 && dst[offset - 1] == '\\') {
+		offset--;
+		return;
+	}
+	txt_add(&eos, 1);
+	abc_parse((char *) dst, fname, linenum);
+	offset = 0;
+}
 
-	sprintf((char *) tmp, "%%@%d", nline);
-	txt_add(tmp, strlen((char *) tmp));
+/* get the ABC version */
+static void get_vers(char *p)
+{
+	int i, j, k;
+
+	i = j = k = 0;
+	if (sscanf(p, "%d.%d.%d", &i, &j, &k) != 3)
+		if (sscanf(p, "%d.%d", &i, &j) != 2)
+			sscanf(p, "%d", &i);
+	parse.abc_vers = (i << 16) + (j << 8) + k;
 }
 
 /* check if the current tune is to be selected */
 static int tune_select(unsigned char *s)
 {
-	struct slre slre;
+	regex_t r;
 	unsigned char *p, *sel;
+	int ret;
 
 	/* if there is a list of tune indexes,
 	 * check the tune index */
@@ -438,75 +451,84 @@ static int tune_select(unsigned char *s)
 			p++;		/* keep the EOL for RE with '\s' */
 		break;
 	}
-//fixme: should compile only one time
-	if (!slre_compile(&slre, (char *) sel))
-		return 0;
-	return slre_match(&slre, (char *) s, p - s, 0);
-}
 
-/* -- init the front-end -- */
-void front_init(int edit,	/* for edition - keep comments */
-		int eol,	/* 0: \n, 1: \r, 2: \r\n */
-		void include_api(unsigned char *fn))
-{
-	keep_comments = edit;
-	txt_add_eol = eol_tb[eol];
-	include_f = include_api;
-	dst = 0;
-	offset = 0;
-	size = 0;
+	ret = p - s;
+	if (ret >= TEX_BUF_SZ - 1) {
+		fprintf(stderr, "Tune header too big for %%%%select\n");
+		return 0;
+	}
+	memcpy(tex_buf, s, ret);
+	tex_buf[ret] = '\0';
+
+	ret = regcomp(&r, (char *) sel,
+				REG_EXTENDED | REG_NEWLINE | REG_NOSUB);
+	if (ret)
+		return 0;
+	ret = regexec(&r, tex_buf, 0, NULL, 0);
+	regfree(&r);
+	return !ret;
 }
 
 /* -- front end parser -- */
-unsigned char *frontend(unsigned char *s,
-			int ftype)
+void frontend(unsigned char *s,
+		int ftype,
+		char *fname,
+		int linenum)
 {
-	unsigned char *p, *q, c, *begin_end;
-	int i, l, state, str_cnv_p, histo, end_len, nline;
+	unsigned char *p, *q, c, *begin_end, sep;
+	int i, l, str_cnv_p, histo, end_len;
 	char prefix_sav[4];
-	int latin_sav;
+	int latin_sav = 0;		/* have C compiler happy */
 
-	begin_end = 0;
+	begin_end = NULL;
 	end_len = 0;
 	histo = 0;
-	state = 0;
-	nline = 0;
-	if (dst != 0)			/* if continuation */
-		offset--;		/* restart before the EOL */
+//	state = 0;
 
-	add_lnum(0);
-	txt_add_eol();
+	if (ftype == FE_ABC
+	 && strncmp((char *) s, "%abc-", 5) == 0) {
+		get_vers((char *) s + 5);
+		while (*s != '\0'
+		    && *s != '\r'
+		    && *s != '\n')
+			s++;
+		if (*s != '\0') {
+			s++;
+			if (s[-1] == '\r' && *s == '\n')
+				s++;
+		}
+		linenum++;
+	}
 
 	/* if unknown encoding, check if latin1 or utf-8 */
 	if (ftype == FE_ABC
-	 && strncmp((char *) s, "%abc-2.1", 8) == 0) {
-		latin = 0;
+	 && parse.abc_vers >= ((2 << 16) | (1 << 8))) {	// if ABC version >= 2.1
+		latin = 0;				// always UTF-8
 	} else {
 		for (p = s; *p != '\0'; p++) {
 			c = *p;
 			if (c == '\\') {
-				if (p[1] == '2') {
-					if (p[2] == '0')	/* accidental */
-						continue;
-					c = 0x80;
-				} else if (p[1] == '3') {
-					c = 0xc0;
-				}
+				if (!isdigit(p[1]))
+					continue;
+				if ((p[1] == '0' || p[1] == '2')
+				 && p[2] == '0')	/* accidental */
+					continue;
+				latin = 1;
+				break;
 			}
 			if (c < 0x80)
 				continue;
-//fixme: problem when two octal values give a UTF-8 character
-			if (c >= 0xc0) {
-				if ((p[1] & 0xc0) == 0x80
-				 || (p[1] == '\\' && p[2] == '2')) {
+			if (c >= 0xc2) {
+				if ((p[1] & 0xc0) == 0x80) {
 					latin = 0;
 					break;
-				 }
+				}
 			}
 			latin = 1;
 			break;
 		}
 	}
+	latin_sav = latin;		/* (have gcc happy) */
 
 	/* scan the file */
 	skip = 0;
@@ -519,6 +541,7 @@ unsigned char *frontend(unsigned char *s,
 		    && *p != '\r'
 		    && *p != '\n') {
 			if (*p == '\\'
+			 || *p == '%'
 			 || (latin > 0 && *p >= 0x80))
 				str_cnv_p = 1;
 			p++;
@@ -529,19 +552,23 @@ unsigned char *frontend(unsigned char *s,
 			if (p[-1] == '\r' && *p == '\n')	/* (DOS) */
 				p++;
 		}
-		nline++;
+		linenum++;
 
+		if (skip) {
+			if (l != 0)
+				goto ignore;
+			skip = 0;
+		}
 		if (begin_end) {
 			if (ftype == FE_FMT) {
 				if (strncmp((char *) s, "end", 3) == 0
 				 && strncmp((char *) s + 3,
 						(char *) begin_end, end_len) == 0) {
-					begin_end = 0;
-					txt_add((unsigned char *) "%%", 2);
-					goto next;
+					begin_end = NULL;
+					goto next_eol;
 				}
 				if (*s == '%')
-					goto next_eol;		/* comment */
+					goto ignore;		/* comment */
 				goto next;
 			}
 			if (*s == '%' && strchr(prefix, s[1])) {
@@ -551,16 +578,13 @@ unsigned char *frontend(unsigned char *s,
 				if (strncmp((char *) q, "end", 3) == 0
 				 && strncmp((char *) q + 3,
 						(char *) begin_end, end_len) == 0) {
-					begin_end = 0;
-					txt_add((unsigned char *) "%%", 2);
-					l -= q - s;
-					s = q;
-					goto next;
+					begin_end = NULL;
+					goto next_eol;
 				}
 			}
 			if (strncmp("ps", (char *) begin_end, end_len) == 0) {
 				if (*s == '%')
-					goto next_eol;		/* comment */
+					goto ignore;		/* comment */
 			} else {
 				if (*s == '%' && strchr(prefix, s[1])) {
 					s += 2;
@@ -570,24 +594,21 @@ unsigned char *frontend(unsigned char *s,
 			goto next;
 		}
 
-		if (skip) {
-			if (l != 0)
-				goto next_eol;
-			skip = 0;
-			txt_add_eol();
-			add_lnum(nline);
-		}
+		while (l > 0 && isspace(s[l - 1]))
+			l--;
 
 		if (l == 0) {			/* empty line */
+			if (ftype == FE_FMT)
+				goto next_eol;
 			switch (state) {
+			default:
+				goto ignore;
 			case 1:
 				fprintf(stderr,
 					"Line %d: Empty line in tune header - K:C added\n",
-					nline);
+					linenum);
 				txt_add((unsigned char *) "K:C", 3);
-				txt_add_eol();
-				txt_add_eol();
-				add_lnum(nline);
+				txt_add_eos(fname, linenum);
 				/* fall thru */
 			case 2:
 				state = 0;
@@ -614,25 +635,20 @@ unsigned char *frontend(unsigned char *s,
 			do {
 				q++;
 			} while (*q == ' ' || *q == '\t');
-			if (*q == '%') {
-				if (keep_comments)
-					txt_add(q, l - (q - s));
-				else if (state != 0)	/* inside tune */
-					txt_add(q, 1);	/* keep a single '%' */
-				goto next_eol;
-			}
+			if (*q == '%')
+				goto ignore;
 		}
 
 		if (ftype == FE_PS) {
 			if (*s == '%')
-				goto next_eol;
+				goto ignore;
 			goto next;
 		}
 
 		/* treat the pseudo-comments */
 		if (ftype == FE_FMT) {
 			if (*s == '%')
-				goto next_eol;
+				goto ignore;
 			goto pscom;
 		}
 		if (*s == 'I' && s[1] == ':') {
@@ -643,20 +659,20 @@ unsigned char *frontend(unsigned char *s,
 				l--;
 			}
 			txt_add((unsigned char *) "%%", 2);
-			goto info;
+			goto pcinfo;
 		}
 		if (*s == '%') {
-			if (!strchr(prefix, s[1])) {		/* pure comment */
-				if (keep_comments
-				 || strncmp((char *) s, "%abc", 4) == 0)
-					txt_add(s, l);
-				else if (state != 0)		/* if not global */
-					txt_add(s, 1);
-				goto next_eol;
-			}
+			if (!strchr(prefix, s[1]))	/* pure comment */
+				goto ignore;
 			s += 2;
 			l -= 2;
-			if (strncmp((char *) s, "abcm2ps ", 8) == 0) {
+			if (strncmp((char *) s, "abc ", 4) == 0) {
+				s += 4;
+				l -= 4;
+				goto info;
+			}
+			if (strncmp((char *) s, "abcm2ps ", 8) == 0
+			 || strncmp((char *) s, "ss-pref ", 8) == 0) {
 				s += 8;
 				l -= 8;
 				while (*s == ' ' || *s == '\t') {
@@ -672,8 +688,11 @@ unsigned char *frontend(unsigned char *s,
 				if (i == 0)
 					prefix[i++] = '%';
 				prefix[i] = '\0';
-				txt_add((unsigned char *) "%", 1);
-				goto next_eol;
+				goto ignore;
+			}
+			if (strncmp((char *) s, "abc-version ", 12) == 0) {
+				get_vers((char *) s + 12);
+				goto ignore;
 			}
 pscom:
 			while (*s == ' ' || *s == '\t') {
@@ -688,7 +707,7 @@ pscom:
 				end_len = q - begin_end;
 				goto next;
 			}
-info:
+pcinfo:
 			if (strncmp((char *) s, "encoding ", 9) == 0
 			 || strncmp((char *) s, "abc-charset ", 12) == 0) {
 				if (*s == 'e')
@@ -731,10 +750,9 @@ info:
 				}
 				goto next;
 			}
-			if (include_f
-			 && (strncmp((char *) s, "format ", 7) == 0
-			  || strncmp((char *) s, "abc-include ", 12) == 0)) {
-				unsigned char sep;
+			if (strncmp((char *) s, "format ", 7) == 0
+			  || strncmp((char *) s, "abc-include ", 12) == 0) {
+				int skip_sav;
 
 				if (*s == 'f')
 					s += 7;
@@ -746,20 +764,21 @@ info:
 				while (*q != '\0'
 				    && *q != '%'
 				    && *q != '\n'
-				    && *q != '\n'
 				    && *q != '\r')
 					q++;
 				while (q[-1] == ' ')
 					q--;
 				sep = *q;
 				*q = '\0';
-				offset--;		/* remove one % */
-				dst[offset - 1] = '\0';	/* replace the other % by EOS */
-				include_f(s);
-				offset--;		/* remove the EOS */
+				skip_sav = skip;
+//fixme: pb when different encoding in included file: != behaviour .fmt or .abc...
+//				latin_sav = latin;
+				offset = 0;
+				include_file(s);
+//				latin = latin_sav;
+				skip = skip_sav;
 				*q = sep;
-				add_lnum(nline);
-				goto next_eol;
+				goto ignore;
 			}
 			if (strncmp((char *) s, "select", 6) == 0) {
 				s += 6;
@@ -786,22 +805,19 @@ info:
 					selection = NULL;
 				}
 				if (q != s) {
-					unsigned char sep;
-
 					sep = *q;
 					*q = '\0';
 					selection = (unsigned char *) strdup((char *) s);
 					*q = sep;
 				}
-				offset--;		/* remove one % */
-				goto next_eol;
+				offset = 0;
+				goto ignore;
 			}
 			goto next;
 		}
-		if (begin_end)
-			goto next;
 
 		/* treat the information fields */
+info:
 		if (s[1] == ':' && (isalpha(*s) || *s == '+')) {
 			c = *s;
 			switch (c) {
@@ -814,24 +830,23 @@ info:
 				case 1:
 					fprintf(stderr,
 						"Line %d: X: found in tune header - K:C added\n",
-						nline);
+						linenum);
 					txt_add((unsigned char *) "K:C", 3);
-					txt_add_eol();
-					txt_add_eol();	/* empty line */
-					add_lnum(nline);
-					txt_add_eol();
+					txt_add_eos(fname, linenum);
+					txt_add_eos(fname, linenum);	/* empty line */
 					break;
 				case 2:
-					txt_add_eol();	/* no empty line - minor error */
+					txt_add_eos(fname, linenum);	/* no empty line - minor error */
 					break;
 				}
-				if (selection)
+				if (selection) {
 					skip = !tune_select(s);
-				if (!skip) {
-					state = 1;
-					strcpy(prefix_sav, prefix);
-					latin_sav = latin;
+					if (skip)
+						goto ignore;
 				}
+				state = 1;
+				strcpy(prefix_sav, prefix);
+				latin_sav = latin;
 				break;
 			case 'U':
 				break;
@@ -840,11 +855,8 @@ info:
 				break;
 			default:
 				if (state == 0			/* if global */
-				 && strchr("dKPQsVWw", *s) != 0) {
-					if (keep_comments)
-						txt_add(s, l);
-					goto next_eol;		/* ignore */
-				}
+				 && strchr("dKPQsVWw", *s) != NULL)
+					goto ignore;
 				if (*s == 'K')
 					state = 2;
 				break;
@@ -862,93 +874,29 @@ info:
 
 		/* treat the music lines */
 		if (state == 0)				/* if not in tune */
-			goto next_eol;			/* ignore */
-		if (!str_cnv_p)
-			goto next;
-		str_cnv_p = 0;
-		for (i = 0; i < l; i++) {
-			if (s[i] != '"')
-				continue;
-			i++;
-			txt_add(s, i);
-			s += i;
-			l -= i;
-			for (i = 0; i < l; i++) {
-				if (s[i] == '"' && s[i - 1] != '\\')
-					break;
-			}
-//fixme: if i == l, no end of string - error
-			txt_add_cnv(s, i);
-			s += i;
-			l -= i;
-			i = 0;
-		}
+			goto ignore;
 next:
 		if (str_cnv_p)
-			txt_add_cnv(s, l);
+			txt_add_cnv(s, l, !begin_end);
 		else
 			txt_add(s, l);
+		if (begin_end)
+			txt_add((unsigned char *) "\n", 1);
+		else
 next_eol:
-		txt_add_eol();
+			txt_add_eos(fname, linenum);
+ignore:
 		s = p;
 	}
-	txt_add((unsigned char *) "", 1);			/* EOS */
-	return dst;
+	if (begin_end)
+		fprintf(stderr,
+			"Line %d: No %%%%end after %%%%begin\n",
+			linenum);
+	if (ftype == FE_FMT)
+		return;
+	if (state == 1)
+		fprintf(stderr,
+			"Line %d: Unexpected EOF in header definition\n",
+			linenum);
+	abc_eof();
 }
-
-#ifdef MAIN
-static void usage(void)
-{
-	printf("ABC frontend\n"
-		"Usage: abcmfe file\n");
-	exit(EXIT_SUCCESS);
-}
-
-int main(int argc, char **argv)
-{
-	char *p, *file, *result;
-	char *fname = 0;
-	FILE *fin;
-	size_t fsize;
-
-	while (--argc > 0) {
-		argv++;
-		p = *argv;
-		if (*p == '-')
-			continue;
-		fname = p;
-	}
-	if (fname == 0)
-		usage();
-
-	if ((fin = fopen(fname, "rb")) == 0) {
-		fprintf(stderr, "Cannot open '%s'\n", fname);
-		exit(EXIT_FAILURE);
-	}
-
-	if (fseek(fin, 0L, SEEK_END) < 0) {
-		fclose(fin);
-		return 0;
-	}
-	fsize = ftell(fin);
-	rewind(fin);
-	if ((file = malloc(fsize + 2)) == 0) {
-		fclose(fin);
-		return 0;
-	}
-
-	if (fread(file, 1, fsize, fin) != fsize) {
-		fclose(fin);
-		free(file);
-		return 0;
-	}
-	file[fsize] = '\0';
-	fclose(fin);
-
-	front_init(0, 0, 0);
-	result = (char *) frontend((unsigned char *) file, 0);
-
-	fputs(result, stdout);
-	return EXIT_SUCCESS;
-}
-#endif
