@@ -3,29 +3,21 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2014 Jean-François Moine
+ * Copyright (C) 1998-2017 Jean-François Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
 #include <string.h>
 
-#include "abc2ps.h"
+#include "abcm2ps.h"
 
 static char ps_head[] =
+	"/xydef{/y exch def/x exch def}!\n"
 	"/xymove{/x 2 index def/y 1 index def M}!\n"
 
 	/* str showc - center at current pt */
@@ -42,97 +34,16 @@ static char ps_head[] =
 	"	currentfont/ScaleMatrix get 0 get .8 mul\n"
 	"	4 add rectstroke}!\n"
 
-	/* x y tclef - treble clef */
-	"/utclef{<95200072\n"
-	"	0000ff2e01c2030c\n"
-	"	00ac0056\n"
-	"	0064007f006400f400e00112\n"
-	"	0176011c01bc0056013a0012\n"
-	"	00c8ffde002700120015009a\n"
-	"	0006014f0072017f00f101e8\n"
-	"	0149023f0140026d012f02ba\n"
-	"	00fc029900d1025100d60200\n"
-	"	00e700f500fa008a0107ffc2\n"
-	"	010dff6200f4ff3c00baff3b\n"
-	"	006aff3a003cff98007dffc0\n"
-	"	00d2ffe90102ff5b009cff57\n"
-	"	00b3ff4600f8ff3200f6ffb3\n"
-	"	00ec009200cf010900c4021c\n"
-	"	00c4027600c402be01240304\n"
-	"	015c02bc0163021e013a01e3\n"
-	"	00f001790039013b003b00a7\n"
-	"	0044000e00cfffee01370022\n"
-	"	018d0063015400e200e700d2\n"
-	"	00a000c6007e008f00ac0056\n"
-	"	><0b000132050a>}cvlit def\n"
-	"/tclef{gsave T -10 0 T .045 dup scale utclef ufill grestore}!\n"
-	"/stclef{gsave T -10 0 T .037 dup scale utclef ufill grestore}!\n"
+	/* x y oct - upper/lower clef '8' */
+	"/oct{/Times-Roman 12 selectfont M(8)show}!\n"
 
-	/* x y octu - upper '8' */
-	"/octu{/Times-Roman 12 selectfont M -2.5 0 RM(8)show}!\n"
-	/* x y octl - lower '8' */
-	"/octl{/Times-Roman 12 selectfont M -3.5 0 RM(8)show}!\n"
-
-	/* x y bclef - bass clef */
-	"/ubclef{<95200046\n"
-	"	00000050019a0244\n"
-	"	00010057\n"
-	"	007d007a00df00a500ff0143\n"
-	"	012a022700580239003f01aa\n"
-	"	007a01fa00dc0194009b015c\n"
-	"	005d012d00280172003101b4\n"
-	"	00460241013f023c01430180\n"
-	"	014200d100d9007800010057\n"
-	"	01660151\n"
-	"	016601750199017301990151\n"
-	"	0199012c0166012d01660151\n"
-	"	016401d2\n"
-	"	016401f6019701f4019701d2\n"
-	"	019701ac016401ad016401d2\n"
-	"	><0b000126050a0122050a0122050a>}cvlit def\n"
-	"/bclef{gsave T -10 0 T .045 dup scale ubclef ufill grestore}!\n"
-	"/sbclef{gsave T -10 3 T .037 dup scale ubclef ufill grestore}!\n"
-
-	/* x y cclef */
-	"/ucclef{<95200066\n"
-	"	006effbe01e70256\n"
-	"	00d10108\n"
-	"	00d10002\n"
-	"	00c40002\n"
-	"	00c40213\n"
-	"	00d10213\n"
-	"	00d10113\n"
-	"	00ea012700fa013701100180\n"
-	"	011e0161011d014d0148013a\n"
-	"	01a2011801a80244011f01f3\n"
-	"	015301e0013a01a3011401a6\n"
-	"	00ba01cc01350256019f01eb\n"
-	"	01e7019c01a000fa01190131\n"
-	"	0109010a\n"
-	"	011900e4\n"
-	"	01a0011b01e70079019f002a\n"
-	"	0135ffbe00ba00490114006f\n"
-	"	013a007201530035011f0022\n"
-	"	01a8ffd101a200fd014800db\n"
-	"	011d00c8011b00bd0110009b\n"
-	"	00fa00e400ea00f400d10108\n"
-	"	006e0213\n"
-	"	00a70213\n"
-	"	00a70002\n"
-	"	006e0002\n"
-	"	006e0213\n"
-	"	><0b000125032605220326050a0124030a>}cvlit def\n"
-	"/cclef{gsave T -12 0 T .045 dup scale ucclef ufill grestore}!\n"
-	"/scclef{gsave T -12 2 T .037 dup scale ucclef ufill grestore}!\n"
-
-	/* x y pclef */
-	"/pclef{	exch 2.7 sub exch 2 add 5.4 20\n"
-	"	1.4 SLW rectstroke}!\n"
-	"/spclef{pclef}!\n"
+	/* x y octu - upper '8' - compatibility */
+	"/octu{oct}!\n"
+	/* x y octl - lower '8'  - compatibility */
+	"/octl{oct}!\n"
 
 	/* t dx dy x y bm - beam, depth t */
-	"/bm{	M 3 copy RL neg 0 exch RL\n"
-	"	neg exch neg exch RL 0 exch RL fill}!\n"
+	"/bm{	M 3 copy RL neg 0 exch RL neg exch neg exch RL 0 exch RL fill}!\n"
 
 	/* str x y bnum - tuplet number / ratio */
 	"/bnum{M/Times-Italic 12 selectfont showc}!\n"
@@ -146,92 +57,9 @@ static char ps_head[] =
 	"/tubr{3 sub M 0 3 RL RL 0 -3 RL dlw stroke}!\n"
 	"/tubrl{3 add M 0 -3 RL RL 0 3 RL dlw stroke}!\n"
 
-	/* x y r00 - longa rest */
-	"/r00{	xymove\n"
-	"	-1.5 -6 RM currentpoint 3 12 rectfill}!\n"
-
-	/* x y r0 - breve rest */
-	"/r0{	xymove\n"
-	"	-1.5 0 RM currentpoint 3 6 rectfill}!\n"
-
-	/* x y r1 - rest */
-	"/r1{	xymove\n"
-	"	-3.5 3 RM currentpoint 7 3 rectfill}!\n"
-
-	/* x y r2 - half rest */
-	"/r2{	xymove\n"
-	"	-3.5 0 RM currentpoint 7 3 rectfill}!\n"
-
-	/* x y r4 - quarter rest */
-	"/r4{	xymove\n"
-	"	-1 8.5 RM\n"
-	"	3.6 -5.1 RL\n"
-	"	-2.1 -5.2 RL\n"
-	"	2.2 -4.3 RL\n"
-	"	-2.6 2.3 -5.1 0 -2.4 -2.6 RC\n"
-	"	-4.8 3 -1.5 6.9 1.4 4.1 RC\n"
-	"	-3.1 4.5 RL\n"
-	"	1.9 5.1 RL\n"
-	"	-1.5 3.5 RL\n"
-	"	fill}!\n"
-
-	/* 1/8 .. 1/64 rest element */
-	"/r8e{	-1.5 -1.5 -2.4 -2 -3.6 -2 RC\n"
-	"	2.4 2.8 -2.8 4 -2.8 1.2 RC\n"
-	"	0 -2.7 4.3 -2.4 5.9 -0.6 RC\n"
-	"	fill}!\n"
-
-	/* x y r8 - eighth rest */
-	"/r8{	xymove\n"
-	"	.5 SLW 3.3 4 RM\n"
-	"	-3.4 -9.6 RL stroke\n"
-	"	x y M 3.4 4 RM r8e}!\n"
-
-	/* x y r16 - 16th rest */
-	"/r16{	xymove\n"
-	"	.5 SLW 3.3 4 RM\n"
-	"	-4 -15.6 RL stroke\n"
-	"	x y M 3.4 4 RM r8e\n"
-	"	x y M 1.9 -2 RM r8e}!\n"
-
-	/* x y r32 - 32th rest */
-	"/r32{	xymove\n"
-	"	.5 SLW 4.8 10 RM\n"
-	"	-5.5 -21.6 RL stroke\n"
-	"	x y M 4.9 10 RM r8e\n"
-	"	x y M 3.4 4 RM r8e\n"
-	"	x y M 1.9 -2 RM r8e}!\n"
-
-	/* x y r64 - 64th rest */
-	"/r64{	xymove\n"
-	"	.5 SLW 4.8 10 RM\n"
-	"	-7 -27.6 RL stroke\n"
-	"	x y M 4.9 10 RM r8e\n"
-	"	x y M 3.4 4 RM r8e\n"
-	"	x y M 1.9 -2 RM r8e\n"
-	"	x y M 0.4 -8 RM r8e}!\n"
-
-	/* x y r128 - 128th rest */
-	"/r128{	xymove\n"
-	"	.5 SLW 5.8 16 RM\n"
-	"	-8.5 -33.6 RL stroke\n"
-	"	x y M 5.9 16 RM r8e\n"
-	"	x y M 4.4 10 RM r8e\n"
-	"	x y M 2.9 4 RM r8e\n"
-	"	x y M 1.4 -2 RM r8e\n"
-	"	x y M -0.1 -8 RM r8e}!\n"
-
 	/* dx dy dt - dot relative to head */
 	"/dt{x y M RM currentpoint 1.2 0 360 arc fill}!\n"
 
-	/* x y hld - fermata */
-	"/hld{	1.5 add 2 copy 1.5 add M currentpoint 1.3 0 360 arc\n"
-	"	M -7.5 0 RM\n"
-	"	0 11.5 15 11.5 15 0 RC\n"
-	"	-0.25 0 RL\n"
-	"	-1.25 9 -13.25 9 -14.5 0 RC\n"
-	"	fill}!\n"
-
 	/* x y dnb - down bow */
 	"/dnb{	dlw M -3.2 2 RM\n"
 	"	0 7.2 RL\n"
@@ -256,11 +84,10 @@ static char ps_head[] =
 	"	-5 -8.5 -5.5 4.5 -10 -2 RC fill}!\n"
 
 	/* x y stc - staccato mark */
-	"/stc{3 add M currentpoint 1.2 0 360 arc fill}!\n"
+	"/stc{	3 add M currentpoint 1.2 0 360 arc fill}!\n"
 
 	/* x y emb - emphasis bar */
-	"/emb{	1.2 SLW 1 setlinecap M\n"
-	"	-2.5 3 RM 5 0 RL stroke 0 setlinecap}!\n"
+	"/emb{	1.2 SLW 1 setlinecap M -2.5 3 RM 5 0 RL stroke 0 setlinecap}!\n"
 
 	/* x y cpu - roll sign above head */
 	"/cpu{	M -6 0 RM\n"
@@ -273,19 +100,7 @@ static char ps_head[] =
 	"	-2.1 -5 -5.4 -6.8 -7.6 -6 RC fill}!\n"
 
 	/* x y trl - trill sign */
-	"/trl{	/Times-BoldItalic 16 selectfont\n"
-	"	M -4 2 RM(tr)show}!\n"
-
-	/* x y umrd - upper mordent */
-	"/umrd{	4 add M\n"
-	"	2.2 2.2 RL 2.1 -2.9 RL 0.7 0.7 RL\n"
-	"	-2.2 -2.2 RL -2.1 2.9 RL -0.7 -0.7 RL\n"
-	"	-2.2 -2.2 RL -2.1 2.9 RL -0.7 -0.7 RL\n"
-	"	2.2 2.2 RL 2.1 -2.9 RL 0.7 0.7 RL fill}!\n"
-
-	/* x y lmrd - lower mordent */
-	"/lmrd{	2 copy umrd M\n"
-	"	.6 SLW 0 8 RL stroke}!\n"
+	"/trl{	/Times-BoldItalic 16 selectfont M -4 2 RM(tr)show}!\n"
 
 	/* str x y fng - finger (0-5) */
 	"/fng{/Bookman-Demi 8 selectfont M -3 1 RM show}!\n"
@@ -293,6 +108,9 @@ static char ps_head[] =
 	/* str x y dacs - D.C. / D.S. */
 	"/dacs{/Times-Roman 16 selectfont 3 add M showc}!\n"
 
+	/* str x y crdc - italic annotations */
+	"/crdc{/Times-Italic 14 selectfont 5 add M show}!\n"
+
 	/* x y brth - breath */
 	"/brth{/Times-BoldItalic 30 selectfont 6 add M(,)show}!\n"
 
@@ -305,26 +123,6 @@ static char ps_head[] =
 	"	/Times-BoldItalic 16 selectfont(f)show\n"
 	"	/Times-Italic 14 selectfont(z)show}!\n"
 
-	/* x y coda - coda */
-	"/coda{	1 SLW 2 add 2 copy M 0 20 RL\n"
-	"	2 copy M -10 10 RM 20 0 RL stroke\n"
-	"	10 add 6 0 360 arc 1.7 SLW stroke}!\n"
-
-	/* x y sgno - segno */
-	"/sgno{	M 0 3 RM currentpoint currentpoint currentpoint\n"
-	"	1.5 -1.7 6.4 0.3 3 3.7 RC\n"
-	"	-10.4 7.8 -8 10.6 -6.5 11.9 RC\n"
-	"	4 1.9 5.9 -1.7 4.2 -2.6 RC\n"
-	"	-1.3 -0.7 -2.9 1.3 -0.7 2 RC\n"
-	"	-1.5 1.7 -6.4 -0.3 -3 -3.7 RC\n"
-	"	10.4 -7.8 8 -10.6 6.5 -11.9 RC\n"
-	"	-4 -1.9 -5.9 1.7 -4.2 2.6 RC\n"
-	"	1.3 0.7 2.9 -1.3 0.7 -2 RC\n"
-	"	fill\n"
-	"	M 0.8 SLW -6 1.2 RM 12.6 12.6 RL stroke\n"
-	"	7 add exch 6 sub exch 1.2 0 360 arc fill\n"
-	"	8 add exch 6 add exch 1.2 0 360 arc fill}!\n"
-
 	/* w x y cresc - crescendo */
 	"/cresc{	1 SLW M dup 5 RM\n"
 	"	defl 1 and 0 eq\n"
@@ -339,31 +137,33 @@ static char ps_head[] =
 	"	{dup 2.2 RL 0 3.6 RM neg 2.2 RL}\n"
 	"	ifelse stroke}!\n"
 
+	// w x y o8va - ottava
+	"/o8va{	M\n"
+	"	defl 1 and 0 eq\n"
+	"	{14 sub currentpoint\n"
+	"	-5 0 RM /Times-BoldItalic 12 selectfont(8)show\n"
+	"	0 4 RM /Times-BoldItalic 10 selectfont(va)show\n"
+	"	M 14 0 RM}if 0 6 RM\n"
+	"	[6] 0 setdash 5 sub 0 RL currentpoint stroke [] 0 setdash\n"
+	"	M defl 2 and 0 eq\n"
+	"	{0 -6 RL stroke}if}!\n"
+
+	// w x y o8vb - ottava bassa
+	"/o8vb{	M\n"
+	"	defl 1 and 0 eq\n"
+	"	{8 sub currentpoint\n"
+	"	-5 0 RM /Times-BoldItalic 12 selectfont(8)show\n"
+	"	0 4 RM /Times-BoldItalic 10 selectfont(vb)show\n"
+	"	M 8 0 RM}if\n"
+	"	[6] 0 setdash 5 sub 0 RL currentpoint stroke [] 0 setdash\n"
+	"	M defl 2 and 0 eq\n"
+	"	{0 6 RL stroke}if}!\n"
+
 	/* x y dplus - plus */
-	"/dplus{	1.2 SLW 0.5 add M 0 6 RL\n"
-	"	-3 -3 RM 6 0 RL stroke}!\n"
-
-	/* x y accent - accent */
-	"/accent{1.2 SLW M -4 0 RM\n"
-	"	8 2 RL -8 2 RL stroke}!\n"
-
-	/* x y turn - turn */
-	"/turn{	M 5.2 8 RM\n"
-	"	1.4 -0.5 0.9 -4.8 -2.2 -2.8 RC\n"
-	"	-4.8 3.5 RL\n"
-	"	-3 2 -5.8 -1.8 -3.6 -4.4 RC\n"
-	"	1 -1.1 2 -0.8 2.1 0.1 RC\n"
-	"	0.1 0.9 -0.7 1.2 -1.9 0.6 RC\n"
-	"	-1.4 0.5 -0.9 4.8 2.2 2.8 RC\n"
-	"	4.8 -3.5 RL\n"
-	"	3 -2 5.8 1.8 3.6 4.4 RC\n"
-	"	-1 1.1 -2 0.8 -2.1 -0.1 RC\n"
-	"	-0.1 -0.9 0.7 -1.2 1.9 -0.6 RC\n"
-	"	fill}!\n"
+	"/dplus{	1.2 SLW 0.5 add M 0 6 RL -3 -3 RM 6 0 RL stroke}!\n"
 
 	/* x y trnx - turn with line through it */
-	"/turnx{	2 copy turn M\n"
-	"	.6 SLW 0 1.5 RM 0 9 RL stroke}!\n"
+	"/turnx{	2 copy turn .6 SLW 1.5 add M 0 9 RL stroke}!\n"
 
 	/* x y lphr - longphrase */
 	"/lphr{1.2 SLW M 0 -18 RL stroke}!\n"
@@ -394,6 +194,22 @@ static char ps_head[] =
 	/* h x ylow arp - arpeggio */
 	"/arp{gsave 90 rotate exch neg ltr grestore}!\n"
 
+	/* x2 y2 x1 y1 gliss - line glissando */
+	"/gliss{	gsave 2 copy T\n"
+		"exch 4 -1 roll exch sub 3 1 roll sub\n" // dx dy
+		"2 copy exch atan dup rotate\n"		// dx dy alpha
+		"exch pop cos div\n"			// len
+		"8 0 M 14 sub 0 RL stroke "
+		"grestore}!\n"
+
+	/* x2 y2 x1 y1 glisq - squiggly glissando */
+	"/glisq{	gsave 2 copy T\n"
+		"exch 4 -1 roll exch sub 3 1 roll sub\n" // dx dy
+		"2 copy exch atan dup rotate\n"		// dx dy alpha
+		"exch pop cos div\n"			// len
+		"17 sub 8 -4 ltr "
+		"grestore}!\n"
+
 	/* x y wedge - wedge */
 	"/wedge{1 add M -1.5 5 RL 3 0 RL -1.5 -5 RL fill}!\n"
 
@@ -426,27 +242,10 @@ static char ps_head[] =
 	/* x y hl2 - more longer ledger line */
 	"/hl2{	.7 SLW M -9 0 RM 18 0 RL stroke}!\n"
 
-	/* -- accidentals -- */
-	/* glyphs 1000 */
-	"/usharp{<95200024\n"		/* width 460 */
-	"	003cff42019a02ee\n"
-	"	008802be\n"
-	"	0088ff44\n"
-	"	00a8ff44\n"
-	"	00a802be\n"
-	"	0128ff76\n"
-	"	0148ff76\n"
-	"	014802ee\n"
-	"	012802ee\n"
-	"	004001d0\n"
-	"	0040015c\n"
-	"	019201bc\n"
-	"	01920230\n"
-	"	00400076\n"
-	"	00400002\n"
-	"	01920064\n"
-	"	019200d6\n"
-	"	><0b000123030a0123030a0123030a0123030a>}cvlit def\n"
+	/* ancillary function for grace note accidentals */
+	"/gsc{gsave y T .8 dup scale 0 0}!\n"
+
+	// accidentals for text
 	"/uflat{<95200028\n"		/* width 400 */
 	"	0064000001b802ee\n"
 	"	006402ea\n"
@@ -478,6 +277,25 @@ static char ps_head[] =
 	"	01160190\n"
 	"	011600d4\n"
 	"	><0b00012a030a0123030a>}cvlit def\n"
+	"/usharp{<95200024\n"		/* width 460 */
+	"	003cff42019a02ee\n"
+	"	008802be\n"
+	"	0088ff44\n"
+	"	00a8ff44\n"
+	"	00a802be\n"
+	"	0128ff76\n"
+	"	0148ff76\n"
+	"	014802ee\n"
+	"	012802ee\n"
+	"	004001d0\n"
+	"	0040015c\n"
+	"	019201bc\n"
+	"	01920230\n"
+	"	00400076\n"
+	"	00400002\n"
+	"	01920064\n"
+	"	019200d6\n"
+	"	><0b000123030a0123030a0123030a0123030a>}cvlit def\n"
 	"/udblesharp{<95200046\n"	/* width 460 */
 	"	003c006e019001c2\n"
 	"	00f0011a\n"
@@ -522,19 +340,6 @@ static char ps_head[] =
 	"	016201860126014e01260128\n"
 	"	><0b000123030a0105050105050a0123030a0105050105050a>}cvlit def\n"
 
-	/* x y sh0 - sharp sign */
-	"/sh0{	gsave T -4 -5 T .018 dup scale usharp ufill grestore}!\n"
-	/* x y ft0 - flat sign */
-	"/ft0{	gsave T -3.5 -3.5 T .018 dup scale uflat ufill grestore}!\n"
-	/* x y nt0 - natural sign */
-	"/nt0{	gsave T -3 -5 T .018 dup scale unat ufill grestore}!\n"
-	/* x y dsh0 - double sharp */
-	"/dsh0{	gsave T -4 -5 T .018 dup scale udblesharp ufill grestore}!\n"
-	/* x y dft0 - double flat sign */
-	"/dft0{	gsave T -4 -3.5 T .018 dup scale udbleflat ufill grestore}!\n"
-	/* ancillary function for grace note accidentals */
-	"/gsc{gsave y T .7 dup scale 0 0}!\n"
-
 	/* some microtone accidentals */
 	/* 1/4 ton sharp */
 	"/sh1{	gsave T .9 SLW\n"
@@ -556,8 +361,8 @@ static char ps_head[] =
 	"/ftx{	-1.4 2.7 RM\n"
 	"	5.7 3.1 5.7 -3.6 0 -6.7 RC\n"
 	"	3.9 4 4 7.6 0 5.8 RC\n"
-	"	currentpoint fill M\n"
-	"	dlw 0 7.1 RM 0 -12.4 RL stroke}!\n"
+	"	currentpoint fill 7.1 add M\n"
+	"	dlw 0 -12.4 RL stroke}!\n"
 	/* 3/4 ton flat */
 	"/ft513{2 copy gsave -1 1 scale exch neg 3 add exch M ftx grestore\n"
 	"	M 1.5 0 RM ftx}!\n"
@@ -567,13 +372,75 @@ static char ps_head[] =
 	"/ft4tb[/.notdef/ft1/ft0/ft513/dft0]def\n"
 	"/ft4{ft4tb exch get cvx exec}!\n"
 
+	/* -- bars -- */
+	/* h x y bar - thin bar */
+	"/bar{M 1 SLW 0 exch RL stroke}!\n"
+	/* h x y dotbar - dotted bar */
+	"/dotbar{[5] 0 setdash bar [] 0 setdash}!\n"
+	/* h x y thbar - thick bar */
+	"/thbar{3 -1 roll 3 exch rectfill}!\n"
+	/* x y rdots - repeat dots */
+	"/rdots{	2 copy 9 add M currentpoint 1.2 0 360 arc\n"
+	"	15 add M currentpoint 1.2 0 360 arc fill}!\n"
+
+	/* x y xxsig - old time signatures ('o', 'o.', 'c' 'c.') */
+	"/pmsig{0.3 SLW 12 add M currentpoint 5 0 360 arc stroke}!\n"
+	"/pMsig{2 copy pmsig 12 add M currentpoint 1.3 0 360 arc fill}!\n"
+	"/imsig{0.3 SLW 12 add 2 copy 5 add M 5 90 270 arc stroke}!\n"
+	"/iMsig{2 copy imsig 12 add M currentpoint 1.3 0 360 arc fill}!\n"
+
+	/* (top) (bot) x y tsig - time signature */
+	"/tsig{	1 add M gsave/Times-Bold 16 selectfont 1.2 1 scale\n"
+	"	currentpoint 3 -1 roll showc\n"
+	"	12 add M showc grestore}!\n"
+
+	/* (meter) x y stsig - single time signature */
+	"/stsig{	7 add M gsave/Times-Bold 18 selectfont 1.2 1 scale\n"
+	"	showc grestore}!\n"
+
+	/* l x sep0 - hline separator */
+	"/sep0{	dlw 0 M 0 RL stroke}!\n"
+
+	/* h x y bracket */
+	"/bracket{M -5 2 RM currentpoint\n"
+	"	-1.7 2 RM 10.5 -1 12 4.5 12 3.5 RC\n"
+	"	0 -1 -3.5 -5.5 -8.5 -5.5 RC fill\n"
+	"	3 SLW 2 add M\n"
+	"	0 exch neg 8 sub RL currentpoint stroke\n"
+	"	M -1.7 0 RM\n"
+	"	10.5 1 12 -4.5 12 -3.5 RC\n"
+	"	0 1 -3.5 5.5 -8.5 5.5 RC fill}!\n"
+
+	/* x y srep - sequence repeat */
+	"/srep{	M -1 -6 RM 11 12 RL 3 0 RL -11 -12 RL -3 0 RL fill}!\n"
+
+	/* str dy bracket_type dx x y repbra - repeat bracket */
+	"/repbra{gsave dlw T 0 -20 M\n"
+	"	0 20 3 index 1 and 1 eq{RL}{RM}ifelse 0\n"
+	"	RL 2 and 2 eq{0 -20 RL}if stroke\n"
+	"	4 exch M show grestore}!\n"
+
+	/* pp2x pp1x p1 pp1 pp2 p2 p1 SL - slur / tie */
+	"/SL{M RC RL RC closepath fill}!\n"
+
+	/* pp2x pp1x p1 pp1 pp2 p2 p1 dSL - dotted slur / tie */
+	"/dSL{	M [4] 0 setdash .8 SLW RC stroke [] 0 setdash}!\n"
+
+	/* -- text -- */
+	"/strw{	gsave 0 -2000 M/strop/show load def 1 setgray str\n"
+	"	0 setgray currentpoint pop/w exch def grestore}!\n"
+	"/jshow{w 0 32 4 -1 roll widthshow}!\n"
+	"/strop/show load def\n"
+	"/arrayshow{{dup type/stringtype eq{strop}{glyphshow}ifelse}forall}def\n"
+
 	/* str gcshow - guitar chord */
 	"/gcshow{show}!\n"
+	"/agcshow{arrayshow}!\n"
 	/* x y w h box - draw a box */
 	"/box{.6 SLW rectstroke}!\n"
-	/* set the start of a guitar chord in a box */
-	"/boxstart{currentpoint pop/x exch def}!\n"
-	/* mark the end of a guitar chord in a box */
+	/* set the end of a box */
+	"/boxend{currentpoint pop/x exch def}!\n"
+	/* mark the right most  end of a box */
 	"/boxmark{currentpoint pop dup x gt\n"
 	"	{/x exch def}{pop}ifelse}!\n"
 	/* x y dy boxdraw - draw a box around a guitar chord */
@@ -583,6 +450,7 @@ static char ps_head[] =
 
 	/* str anshow - annotation */
 	"/anshow{show}!\n"
+	"/aanshow{arrayshow}!\n"
 
 	/* -- lyrics under notes -- */
 	/* w x y wln - underscore line */
@@ -599,187 +467,28 @@ static char ps_head[] =
 	"	{4 0 RL dup 0 RM}repeat stroke pop}!\n"
 	/* str lyshow - lyrics */
 	"/lyshow{show}!\n"
-
-	/* -- bars -- */
-	/* h x y bar - thin bar */
-	"/bar{M dlw 0 exch RL stroke}!\n"
-	/* h x y dotbar - dotted bar */
-	"/dotbar{[5] 0 setdash bar [] 0 setdash}!\n"
-	/* h x y thbar - thick bar */
-	"/thbar{3 -1 roll 3 exch rectfill}!\n"
-	/* x y rdots - repeat dots */
-	"/rdots{	2 copy 9 add M currentpoint 1.2 0 360 arc\n"
-	"	15 add M currentpoint 1.2 0 360 arc fill}!\n"
-
-	/* x y csig - C timesig */
-	"/csig{	M\n"
-	"	1 17.3 RM\n"
-	"	0.9 0 2.3 -0.7 2.4 -2.2 RC\n"
-	"	-1.2 2 -3.6 -0.1 -1.6 -1.7 RC\n"
-	"	2 -1 3.8 3.5 -0.8 4.7 RC\n"
-	"	-2 0.4 -6.4 -1.3 -5.8 -7 RC\n"
-	"	0.4 -6.4 7.9 -6.8 9.1 -0.7 RC\n"
-	"	-2.3 -5.6 -6.7 -5.1 -6.8 0 RC\n"
-	"	-0.5 4.4 0.7 7.5 3.5 6.9 RC\n"
-	"	fill}!\n"
-
-	/* x y ctsig - C| timesig */
-	"/ctsig{dlw 2 copy csig 4 add M 0 16 RL stroke}!\n"
-
-	/* x y xxsig - old time signatures ('o', 'o.', 'c' 'c.') */
-	"/pmsig{0.3 SLW 12 add M currentpoint 5 0 360 arc stroke}!\n"
-	"/pMsig{2 copy pmsig 12 add M currentpoint 1.3 0 360 arc fill}!\n"
-	"/imsig{0.3 SLW 12 add 2 copy 5 add M 5 90 270 arc stroke}!\n"
-	"/iMsig{2 copy imsig 12 add M currentpoint 1.3 0 360 arc fill}!\n"
-
-	/* (top) (bot) x y tsig - time signature */
-	"/tsig{	M gsave/Times-Bold 16 selectfont 1.2 1 scale\n"
-	"	0 1 RM currentpoint 3 -1 roll showc\n"
-	"	12 add M showc grestore}!\n"
-
-	/* (meter) x y stsig - single time signature */
-	"/stsig{	M gsave/Times-Bold 18 selectfont 1.2 1 scale\n"
-	"	0 7 RM showc grestore}!\n"
-
-	/* width n x y staff - staff with n lines*/
-	"/staff{	dlw M{dup 0 RL dup neg 6 RM}repeat\n"
-	"	pop stroke}!\n"
-
-	/* l x sep0 - hline separator */
-	"/sep0{dlw 0 M 0 RL stroke}!\n"
-
-	"/hbrce{	-2.5 1 RM\n"
-	"	-4.5 -4.6 -7.5 -12.2 -4.4 -26.8 RC\n"
-	"	3.5 -14.3 3.2 -21.7 -2.1 -24.2 RC\n"
-	"	7.4 2.4 7.3 14.2 3.5 29.5 RC\n"
-	"	-2.7 9.5 -1.5 16.2 3 21.5 RC\n"
-	"	fill}!\n"
-	/* h x y brace */
-	"/brace{	gsave T 0 0 M .01 mul 1 exch scale hbrce\n"
-	"	0 -100 M 1 -1 scale hbrce grestore}!\n"
-
-	/* h x y bracket */
-	"/bracket{M -5 2 RM currentpoint\n"
-	"	-1.7 2 RM 10.5 -1 12 4.5 12 3.5 RC\n"
-	"	0 -1 -3.5 -5.5 -8.5 -5.5 RC fill\n"
-	"	3 SLW M 0 2 RM\n"
-	"	0 exch neg 8 sub RL currentpoint stroke\n"
-	"	M -1.7 0 RM\n"
-	"	10.5 1 12 -4.5 12 -3.5 RC\n"
-	"	0 1 -3.5 5.5 -8.5 5.5 RC fill}!\n"
-
-	/* nb_measures x y mrest */
-	"/mrest{	gsave T 1 SLW\n"
-	"	-20 6 M 0 12 RL 20 6 M 0 12 RL stroke\n"
-	"	5 SLW -20 12 M 40 0 RL stroke\n"
-	"	/Times-Bold 15 selectfont 0 28 M showc grestore}!\n"
-
-	/* x y mrep - measure repeat */
-	"/mrep{	2 copy 2 copy\n"
-	"	M -5 15 RM currentpoint 1.4 0 360 arc\n"
-	"	M 5 9 RM currentpoint 1.4 0 360 arc\n"
-	"	M -7 6 RM 11 12 RL 3 0 RL -11 -12 RL -3 0 RL\n"
-	"	fill}!\n"
-
-	/* x y mrep2 - measure repeat 2 times */
-	"/mrep2{	2 copy 2 copy\n"
-	"	M -5 18 RM currentpoint 1.4 0 360 arc\n"
-	"	M 5 6 RM currentpoint 1.4 0 360 arc fill\n"
-	"	M 1.8 SLW\n"
-	"	-7 4 RM 14 10 RL -14 -4 RM 14 10 RL\n"
-	"	stroke}!\n"
-
-	/* x y srep - sequence repeat */
-	"/srep{	M -1 6 RM 11 12 RL 3 0 RL -11 -12 RL -3 0 RL\n"
-	"	fill}!\n"
-
-	/* str dy bracket_type dx x y repbra - repeat bracket */
-	"/repbra{gsave dlw T 0 -20 M\n"
-	"	0 20 3 index 1 ne{RL}{RM}ifelse 0 RL 0 ne{0 -20 RL}if stroke\n"
-	"	4 exch M show grestore}!\n"
-
-	/* pp2x pp1x p1 pp1 pp2 p2 p1 SL - slur / tie */
-	"/SL{M RC RL RC closepath fill}!\n"
-
-	/* pp2x pp1x p1 pp1 pp2 p2 p1 dSL - dotted slur / tie */
-	"/dSL{	M [4] 0 setdash .8 SLW\n"
-	"	RC stroke [] 0 setdash}!\n"
-
-	/* -- text -- */
-	"/strw{	gsave 0 1000 M/strop/show load def 1 setgray str\n"
-	"	0 setgray currentpoint pop/w exch def grestore}!\n"
-	"/jshow{w 0 32 4 -1 roll widthshow}!\n"
-	"/strop/show load def\n"
-	"/arrayshow{{dup type/stringtype eq{strop}{glyphshow}ifelse}forall}def\n"
-
-	/* -- note heads -- */
-	/* x y hd - full head */
-	"/uhd{{	100 -270 640 280\n"
-	"	560 82\n"
-	"	474 267 105 105 186 -80\n"
-	"	267 -265 636 -102 555 82\n"
-	"	}<0b000122050a>}cvlit def\n"
-	"/hd{	/x 2 index def/y 1 index def\n"
-	"	gsave T -7.4 0 T .02 dup scale uhd ufill grestore}!\n"
-	/* x y Hd - open head for half */
-	"/Hd{	xymove\n"
-	"	3 1.6 RM\n"
-	"	-1 1.8 -7 -1.4 -6 -3.2 RC\n"
-	"	1 -1.8 7 1.4 6 3.2 RC\n"
-	"	0.5 0.3 RM\n"
-	"	2 -3.8 -5 -7.6 -7 -3.8 RC\n"
-	"	-2 3.8 5 7.6 7 3.8 RC\n"
-	"	fill}!\n"
-	/* x y HD - open head for whole */
-	"/HD{	xymove\n"
-	"	-2.7 1.4 RM\n"
-	"	1.5 2.8 6.9 0 5.3 -2.7 RC\n"
-	"	-1.5 -2.8 -6.9 0 -5.3 2.7 RC\n"
-	"	8.3 -1.4 RM\n"
-	"	0 1.5 -2.2 3 -5.6 3 RC\n"
-	"	-3.4 0 -5.6 -1.5 -5.6 -3 RC\n"
-	"	0 -1.5 2.2 -3 5.6 -3 RC\n"
-	"	3.4 0 5.6 1.5 5.6 3 RC\n"
-	"	fill}!\n"
-	/* x y HDD - round breve */
-	"/HDD{	dlw HD\n"
-	"	x y M -6 -4 RM 0 8 RL\n"
-	"	12 0 RM 0 -8 RL stroke}!\n"
-	/* x y breve - square breve */
-	"/breve{	xymove\n"
-	"	2.5 SLW -6 -2.7 RM 12 0 RL\n"
-	"	0 5.4 RM -12 0 RL stroke\n"
-	"	dlw x y M -6 -5 RM 0 10 RL\n"
-	"	12 0 RM 0 -10 RL stroke}!\n"
-	/* x y longa */
-	"/longa{	xymove\n"
-	"	2.5 SLW -6 -2.7 RM 12 0 RL\n"
-	"	0 5.4 RM -12 0 RL stroke\n"
-	"	dlw x y M -6 -5 RM 0 10 RL\n"
-	"	12 0 RM 0 -16 RL stroke}!\n"
+	"/alyshow{arrayshow}!\n"
 
 	/* -- default percussion heads -- */
-	/* x y pshhd - percussion sharp head */
-	"/pshhd{/x 2 index def/y 1 index def dsh0}!\n"
 	/* x y pfthd - percussion flat head */
 	"/pfthd{/x 2 index def/y 1 index def dsh0\n"
-	"	.7 SLW x y 4 0 360 arc stroke}!\n"
+	"	.7 SLW x y M x y 4 0 360 arc stroke}!\n"
 	/* same for dble sharp/flat */
 	"/pdshhd{pshhd}!\n"
 	"/pdfthd{pfthd}!\n"
 
 	/* x y ghd - grace note head */
 	"/ghd{	xymove\n"
-	"	1.7 1 RM\n"
-	"	-1 1.7 -4.5 -0.2 -3.4 -2 RC\n"
-	"	1 -1.7 4.5 0.2 3.4 2 RC fill}!\n"
+	"	1.7 1.5 RM\n"
+	"	-1.32 2.31 -5.94 -0.33 -4.62 -2.64 RC\n"
+	"	1.32 -2.31 5.94 0.33 4.62 2.64 RC fill}!\n"
 
 	/* dx dy gua / gda - acciaccatura */
 	"/gua{x y M -1 4 RM RL stroke}!\n"
 	"/gda{x y M -5 -4 RM RL stroke}!\n"
 
 	/* x y ghl - grace note ledger line */
-	"/ghl{	.6 SLW M -3 0 RM 6 0 RL stroke}!\n"
+	"/ghl{	.6 SLW M -3.5 0 RM 7 0 RL stroke}!\n"
 
 	/* x1 y2 x2 y2 x3 y3 x0 y0 gsl - grace note slur */
 	"/gsl{dlw M RC stroke}!\n"
@@ -801,6 +510,342 @@ static char ps_head[] =
 
 	"0 setlinecap 0 setlinejoin\n";
 
+/* PS direct glyphs */
+static char psdgl[] = 
+	"/hbrce{	-2.5 1 RM\n"
+	"	-4.5 -4.6 -7.5 -12.2 -4.4 -26.8 RC\n"
+	"	3.5 -14.3 3.2 -21.7 -2.1 -24.2 RC\n"
+	"	7.4 2.4 7.3 14.2 3.5 29.5 RC\n"
+	"	-2.7 9.5 -1.5 16.2 3 21.5 RC\n"
+	"	fill}!\n"
+	/* h x y brace */
+	"/brace{	gsave T 0 0 M .01 mul 1 exch scale hbrce\n"
+	"	0 -100 M 1 -1 scale hbrce grestore}!\n"
+	/* x y sgno - segno */
+	"/sgno{	3 add M currentpoint currentpoint currentpoint\n"
+	"	1.5 -1.7 6.4 0.3 3 3.7 RC\n"
+	"	-10.4 7.8 -8 10.6 -6.5 11.9 RC\n"
+	"	4 1.9 5.9 -1.7 4.2 -2.6 RC\n"
+	"	-1.3 -0.7 -2.9 1.3 -0.7 2 RC\n"
+	"	-1.5 1.7 -6.4 -0.3 -3 -3.7 RC\n"
+	"	10.4 -7.8 8 -10.6 6.5 -11.9 RC\n"
+	"	-4 -1.9 -5.9 1.7 -4.2 2.6 RC\n"
+	"	1.3 0.7 2.9 -1.3 0.7 -2 RC\n"
+	"	fill\n"
+	"	M 0.8 SLW -6 1.2 RM 12.6 12.6 RL stroke\n"
+	"	7 add exch 6 sub exch 1.2 0 360 arc fill\n"
+	"	8 add exch 6 add exch 1.2 0 360 arc fill}!\n"
+	/* x y coda - coda */
+	"/coda{	1 SLW 2 add 2 copy M 0 20 RL\n"
+	"	2 copy M -10 10 RM 20 0 RL stroke\n"
+	"	10 add 6 0 360 arc 1.7 SLW stroke}!\n"
+	/* x y tclef - treble clef */
+	"/utclef{<95200072\n"
+	"	0000ff2e01c2030c\n"
+	"	00ac0056\n"
+	"	0064007f006400f400e00112\n"
+	"	0176011c01bc0056013a0012\n"
+	"	00c8ffde002700120015009a\n"
+	"	0006014f0072017f00f101e8\n"
+	"	0149023f0140026d012f02ba\n"
+	"	00fc029900d1025100d60200\n"
+	"	00e700f500fa008a0107ffc2\n"
+	"	010dff6200f4ff3c00baff3b\n"
+	"	006aff3a003cff98007dffc0\n"
+	"	00d2ffe90102ff5b009cff57\n"
+	"	00b3ff4600f8ff3200f6ffb3\n"
+	"	00ec009200cf010900c4021c\n"
+	"	00c4027600c402be01240304\n"
+	"	015c02bc0163021e013a01e3\n"
+	"	00f001790039013b003b00a7\n"
+	"	0044000e00cfffee01370022\n"
+	"	018d0063015400e200e700d2\n"
+	"	00a000c6007e008f00ac0056\n"
+	"	><0b000132050a>}cvlit def\n"
+	"/tclef{gsave T -10 -6 T .045 dup scale utclef ufill grestore}!\n"
+	/* x y cclef */
+	"/ucclef{<95200066\n"
+	"	006effbe01e70256\n"
+	"	00d10108\n"
+	"	00d10002\n"
+	"	00c40002\n"
+	"	00c40213\n"
+	"	00d10213\n"
+	"	00d10113\n"
+	"	00ea012700fa013701100180\n"
+	"	011e0161011d014d0148013a\n"
+	"	01a2011801a80244011f01f3\n"
+	"	015301e0013a01a3011401a6\n"
+	"	00ba01cc01350256019f01eb\n"
+	"	01e7019c01a000fa01190131\n"
+	"	0109010a\n"
+	"	011900e4\n"
+	"	01a0011b01e70079019f002a\n"
+	"	0135ffbe00ba00490114006f\n"
+	"	013a007201530035011f0022\n"
+	"	01a8ffd101a200fd014800db\n"
+	"	011d00c8011b00bd0110009b\n"
+	"	00fa00e400ea00f400d10108\n"
+	"	006e0213\n"
+	"	00a70213\n"
+	"	00a70002\n"
+	"	006e0002\n"
+	"	006e0213\n"
+	"	><0b000125032605220326050a0124030a>}cvlit def\n"
+	"/cclef{gsave T -12 -12 T .045 dup scale ucclef ufill grestore}!\n"
+	/* x y bclef - bass clef */
+	"/ubclef{<95200046\n"
+	"	00000050019a0244\n"
+	"	00010057\n"
+	"	007d007a00df00a500ff0143\n"
+	"	012a022700580239003f01aa\n"
+	"	007a01fa00dc0194009b015c\n"
+	"	005d012d00280172003101b4\n"
+	"	00460241013f023c01430180\n"
+	"	014200d100d9007800010057\n"
+	"	01660151\n"
+	"	016601750199017301990151\n"
+	"	0199012c0166012d01660151\n"
+	"	016401d2\n"
+	"	016401f6019701f4019701d2\n"
+	"	019701ac016401ad016401d2\n"
+	"	><0b000126050a0122050a0122050a>}cvlit def\n"
+	"/bclef{gsave T -10 -18 T .045 dup scale ubclef ufill grestore}!\n"
+	/* x y pclef */
+	"/pclef{	exch 2.7 sub exch -9 add 5.4 18 1.4 SLW rectstroke}!\n"
+	"/spclef{pclef}!\n"
+	"/stclef{gsave T -10 -6 T .037 dup scale utclef ufill grestore}!\n"
+	"/scclef{gsave T -12 -10 T .037 dup scale ucclef ufill grestore}!\n"
+	"/sbclef{gsave T -10 -15 T .037 dup scale ubclef ufill grestore}!\n"
+	/* x y csig - C timesig */
+	"/csig{	M\n"
+	"	6 5.3 RM\n"
+	"	0.9 0 2.3 -0.7 2.4 -2.2 RC\n"
+	"	-1.2 2 -3.6 -0.1 -1.6 -1.7 RC\n"
+	"	2 -1 3.8 3.5 -0.8 4.7 RC\n"
+	"	-2 0.4 -6.4 -1.3 -5.8 -7 RC\n"
+	"	0.4 -6.4 7.9 -6.8 9.1 -0.7 RC\n"
+	"	-2.3 -5.6 -6.7 -5.1 -6.8 0 RC\n"
+	"	-0.5 4.4 0.7 7.5 3.5 6.9 RC\n"
+	"	fill}!\n"
+	/* x y ctsig - C| timesig */
+	"/ctsig{dlw 2 copy csig M 5 -8 RM 0 16 RL stroke}!\n"
+	/* x y HDD - round breve */
+	"/HDD{	dlw HD\n"
+	"	x y M -6 -4 RM 0 8 RL\n"
+	"	12 0 RM 0 -8 RL stroke}!\n"
+	/* x y breve - square breve */
+	"/breve{	xymove\n"
+	"	2.5 SLW -6 -2.7 RM 12 0 RL\n"
+	"	0 5.4 RM -12 0 RL stroke\n"
+	"	dlw x y M -6 -5 RM 0 10 RL\n"
+	"	12 0 RM 0 -10 RL stroke}!\n"
+	/* x y HD - open head for whole */
+	"/HD{	xymove\n"
+	"	-2.7 1.4 RM\n"
+	"	1.5 2.8 6.9 0 5.3 -2.7 RC\n"
+	"	-1.5 -2.8 -6.9 0 -5.3 2.7 RC\n"
+	"	8.3 -1.4 RM\n"
+	"	0 1.5 -2.2 3 -5.6 3 RC\n"
+	"	-3.4 0 -5.6 -1.5 -5.6 -3 RC\n"
+	"	0 -1.5 2.2 -3 5.6 -3 RC\n"
+	"	3.4 0 5.6 1.5 5.6 3 RC fill}!\n"
+	/* x y Hd - open head for half */
+	"/Hd{	xymove\n"
+	"	3 1.6 RM\n"
+	"	-1 1.8 -7 -1.4 -6 -3.2 RC\n"
+	"	1 -1.8 7 1.4 6 3.2 RC\n"
+	"	0.5 0.3 RM\n"
+	"	2 -3.8 -5 -7.6 -7 -3.8 RC\n"
+	"	-2 3.8 5 7.6 7 3.8 RC fill}!\n"
+	/* x y hd - full head */
+	"/uhd{{	100 -270 640 280\n"
+	"	560 82\n"
+	"	474 267 105 105 186 -80\n"
+	"	267 -265 636 -102 555 82\n"
+	"	}<0b000122050a>}cvlit def\n"
+	"/hd{	/x 2 index def/y 1 index def\n"
+	"	gsave T -7.4 0 T .02 dup scale uhd ufill grestore}!\n"
+	/* x y ft0 - flat sign */
+	"/ft0{	gsave T -3.5 -3.5 T .018 dup scale uflat ufill grestore}!\n"
+	/* x y nt0 - natural sign */
+	"/nt0{	gsave T -3 -5 T .018 dup scale unat ufill grestore}!\n"
+	/* x y sh0 - sharp sign */
+	"/sh0{	gsave T -4 -5 T .018 dup scale usharp ufill grestore}!\n"
+	/* x y dsh0 - double sharp */
+	"/dsh0{	gsave T -4 -5 T .018 dup scale udblesharp ufill grestore}!\n"
+	/* x y pshhd - percussion sharp head */
+	"/pshhd{/x 2 index def/y 1 index def dsh0}!\n"
+	/* x y dft0 - double flat sign */
+	"/dft0{	gsave T -4 -3.5 T .018 dup scale udbleflat ufill grestore}!\n"
+	/* x y accent - accent */
+	"/accent{1.2 SLW M -4 1 RM 8 2 RL -8 2 RL stroke}!\n"
+	/* x y marcato - accent */
+	"/marcato{M -3 0 RM 3 7 RL 3 -7 RL -1.5 0 RL -1.8 4.2 RL -1.7 -4.2 RL fill}!\n"
+	/* x y hld - fermata */
+	"/hld{	1.5 add 2 copy 1.5 add M currentpoint 1.3 0 360 arc\n"
+	"	M -7.5 0 RM\n"
+	"	0 11.5 15 11.5 15 0 RC\n"
+	"	-0.25 0 RL\n"
+	"	-1.25 9 -13.25 9 -14.5 0 RC\n"
+	"	fill}!\n"
+	/* x y r00 - longa rest */
+	"/r00{	xymove -1.5 -6 RM currentpoint 3 12 rectfill}!\n"
+	/* x y r0 - breve rest */
+	"/r0{	xymove -1.5 0 RM currentpoint 3 6 rectfill}!\n"
+	/* x y r1 - rest */
+	"/r1{	xymove -3.5 3 RM currentpoint 7 3 rectfill}!\n"
+	/* x y r2 - half rest */
+	"/r2{	xymove -3.5 0 RM currentpoint 7 3 rectfill}!\n"
+	/* x y r4 - quarter rest */
+	"/r4{	xymove\n"
+	"	-1 8.5 RM\n"
+	"	3.6 -5.1 RL\n"
+	"	-2.1 -5.2 RL\n"
+	"	2.2 -4.3 RL\n"
+	"	-2.6 2.3 -5.1 0 -2.4 -2.6 RC\n"
+	"	-4.8 3 -1.5 6.9 1.4 4.1 RC\n"
+	"	-3.1 4.5 RL\n"
+	"	1.9 5.1 RL\n"
+	"	-1.5 3.5 RL\n"
+	"	fill}!\n"
+	/* 1/8 .. 1/64 rest element */
+	"/r8e{	-1.5 -1.5 -2.4 -2 -3.6 -2 RC\n"
+	"	2.4 2.8 -2.8 4 -2.8 1.2 RC\n"
+	"	0 -2.7 4.3 -2.4 5.9 -0.6 RC\n"
+	"	fill}!\n"
+	/* x y r8 - eighth rest */
+	"/r8{	xymove\n"
+	"	.5 SLW 3.3 4 RM\n"
+	"	-3.4 -9.6 RL stroke\n"
+	"	x y M 3.4 4 RM r8e}!\n"
+	/* x y r16 - 16th rest */
+	"/r16{	xymove\n"
+	"	.5 SLW 3.3 4 RM\n"
+	"	-4 -15.6 RL stroke\n"
+	"	x y M 3.4 4 RM r8e\n"
+	"	x y M 1.9 -2 RM r8e}!\n"
+	/* x y r32 - 32th rest */
+	"/r32{	xymove\n"
+	"	.5 SLW 4.8 10 RM\n"
+	"	-5.5 -21.6 RL stroke\n"
+	"	x y M 4.9 10 RM r8e\n"
+	"	x y M 3.4 4 RM r8e\n"
+	"	x y M 1.9 -2 RM r8e}!\n"
+	/* x y r64 - 64th rest */
+	"/r64{	xymove\n"
+	"	.5 SLW 4.8 10 RM\n"
+	"	-7 -27.6 RL stroke\n"
+	"	x y M 4.9 10 RM r8e\n"
+	"	x y M 3.4 4 RM r8e\n"
+	"	x y M 1.9 -2 RM r8e\n"
+	"	x y M 0.4 -8 RM r8e}!\n"
+	/* x y r128 - 128th rest */
+	"/r128{	xymove\n"
+	"	.5 SLW 5.8 16 RM\n"
+	"	-8.5 -33.6 RL stroke\n"
+	"	x y M 5.9 16 RM r8e\n"
+	"	x y M 4.4 10 RM r8e\n"
+	"	x y M 2.9 4 RM r8e\n"
+	"	x y M 1.4 -2 RM r8e\n"
+	"	x y M -0.1 -8 RM r8e}!\n"
+	/* x y mrest */
+	"/mrest{	M currentpoint 1 SLW\n"
+	"	-20 -6 RM 0 12 RL 40 0 RM 0 -12 RL stroke\n"
+	"	M 5 SLW -20 0 RM 40 0 RL stroke}!\n"
+	/* x y mrep - measure repeat */
+	"/mrep{	2 copy 2 copy\n"
+	"	M -5 3 RM currentpoint 1.4 0 360 arc\n"
+	"	M 5 -3 RM currentpoint 1.4 0 360 arc\n"
+	"	M -7 -6 RM 11 12 RL 3 0 RL -11 -12 RL -3 0 RL fill}!\n"
+	/* x y mrep2 - measure repeat 2 times */
+	"/mrep2{	2 copy 2 copy\n"
+	"	M -5 6 RM currentpoint 1.4 0 360 arc\n"
+	"	M 5 -6 RM currentpoint 1.4 0 360 arc fill\n"
+	"	M 1.8 SLW\n"
+	"	-7 -8 RM 14 10 RL -14 -4 RM 14 10 RL stroke}!\n"
+	/* x y turn - turn */
+	"/turn{	M 5.2 8 RM\n"
+	"	1.4 -0.5 0.9 -4.8 -2.2 -2.8 RC\n"
+	"	-4.8 3.5 RL\n"
+	"	-3 2 -5.8 -1.8 -3.6 -4.4 RC\n"
+	"	1 -1.1 2 -0.8 2.1 0.1 RC\n"
+	"	0.1 0.9 -0.7 1.2 -1.9 0.6 RC\n"
+	"	-1.4 0.5 -0.9 4.8 2.2 2.8 RC\n"
+	"	4.8 -3.5 RL\n"
+	"	3 -2 5.8 1.8 3.6 4.4 RC\n"
+	"	-1 1.1 -2 0.8 -2.1 -0.1 RC\n"
+	"	-0.1 -0.9 0.7 -1.2 1.9 -0.6 RC\n"
+	"	fill}!\n"
+	/* x y umrd - upper mordent */
+	"/umrd{	4 add M\n"
+	"	2.2 2.2 RL 2.1 -2.9 RL 0.7 0.7 RL\n"
+	"	-2.2 -2.2 RL -2.1 2.9 RL -0.7 -0.7 RL\n"
+	"	-2.2 -2.2 RL -2.1 2.9 RL -0.7 -0.7 RL\n"
+	"	2.2 2.2 RL 2.1 -2.9 RL 0.7 0.7 RL fill}!\n"
+	/* x y lmrd - lower mordent */
+	"/lmrd{	2 copy umrd M .6 SLW 0 8 RL stroke}!\n"
+	// dummy !ped! and !ped-up!
+	"/ped{	/Times-BoldItalic 16 selectfont M -10 2 RM(Ped)show}!\n"
+	"/pedoff{	/Times-BoldItalic 16 selectfont M -4 2 RM(*)show}!\n"
+	/* x y longa */
+	"/longa{	xymove\n"
+	"	2.5 SLW -6 -2.7 RM 12 0 RL\n"
+	"	0 5.4 RM -12 0 RL stroke\n"
+	"	dlw x y M -6 -5 RM 0 10 RL\n"
+	"	12 0 RM 0 -16 RL stroke}!\n";
+
+/* PS font glyphs */
+static char psfgl[] =
+	"/musgly{music 24 selectfont glyphshow}!\n"
+	"/brace{gsave\n"
+	"	T -7.5 0 M -.042 mul 3 exch scale/uniE000 musgly\n"
+	"	grestore}!\n"
+	"/sgno{	M -6 4 RM/uniE047 musgly}!\n"
+	"/coda{	M -12 6 RM/uniE048 musgly}!\n"
+	"/tclef{M -8 0 RM/uniE050 musgly}!\n"
+	"/cclef{M -8 0 RM/uniE05C musgly}!\n"
+	"/bclef{M -8 0 RM/uniE062 musgly}!\n"
+	"/pclef{M -6 0 RM/uniE069 musgly}!\n"
+	"/stclef{M -8 0 RM/uniE07A musgly}!\n"
+	"/scclef{M -8 0 RM/uniE07B musgly}!\n"
+	"/sbclef{M -7 0 RM/uniE07C musgly}!\n"
+	"/csig{	M 0 0 RM/uniE08A musgly}!\n"
+	"/ctsig{M 0 0 RM/uniE08B musgly}!\n"
+	"/HDD{	xymove -7 0 RM/uniE0A0 musgly}!\n"
+	"/breve{xymove -6 0 RM/uniE0A1 musgly}!\n"
+	"/HD{	xymove -5.2 0 RM/uniE0A2 musgly}!\n"
+	"/Hd{	xymove -3.8 0 RM/uniE0A3 musgly}!\n"
+	"/hd{	xymove -3.7 0 RM/uniE0A4 musgly}!\n"
+	"/ft0{	M -3 0 RM/uniE260 musgly}!\n"
+	"/nt0{	M -2 0 RM/uniE261 musgly}!\n"
+	"/sh0{	M -3 0 RM/uniE262 musgly}!\n"
+	"/dsh0{	M -3 0 RM/uniE263 musgly}!\n"
+	"/pshhd{xymove -3 0 RM/uniE263 musgly}!\n"
+	"/dft0{	M -3 0 RM/uniE264 musgly}!\n"
+	"/accent{M -3 0 RM/uniE4A0 musgly}!\n"
+	"/marcato{M -3 0 RM/uniE4AC musgly}!\n"
+	"/hld{	M -7 0 RM/uniE4C0 musgly}!\n"
+	"/r00{	xymove -1.5 0 RM/uniE4E1 musgly}!\n"
+	"/r0{	xymove -1.5 0 RM/uniE4E2 musgly}!\n"
+	"/r1{	xymove -3.5 6 RM/uniE4E3 musgly}!\n"
+	"/r2{	xymove -3.2 0 RM/uniE4E4 musgly}!\n"
+	"/r4{	xymove -3 0 RM/uniE4E5 musgly}!\n"
+	"/r8{	xymove -3 0 RM/uniE4E6 musgly}!\n"
+	"/r16{	xymove -4 0 RM/uniE4E7 musgly}!\n"
+	"/r32{	xymove -4 0 RM/uniE4E8 musgly}!\n"
+	"/r64{	xymove -4 0 RM/uniE4E9 musgly}!\n"
+	"/r128{	xymove -4 0 RM/uniE4EA musgly}!\n"
+	"/mrest{M -10 0 RM/uniE4EE musgly}!\n"
+	"/mrep{	M -6 0 RM/uniE500 musgly}!\n"
+	"/mrep2{M -9 0 RM/uniE501 musgly}!\n"
+	"/turn{	M -4 0 RM/uniE567 musgly}!\n"
+	"/umrd{	M -7 2 RM/uniE56C musgly}!\n"
+	"/lmrd{	M -7 2 RM/uniE56D musgly}!\n"
+	"/ped{	M -10 0 RM/uniE650 musgly}!\n"
+	"/pedoff{M -6 0 RM/uniE655 musgly}!\n"
+	"/longa{xymove -6 0 RM/uniE95C musgly}!\n";
+
 /* -- define a font -- */
 void define_font(char name[],
 		 int num,
@@ -817,7 +862,23 @@ void define_font(char name[],
 /* -- output the symbol definitions -- */
 void define_symbols(void)
 {
+	char *p, *q, *r;
+
+	p = cfmt.musicfont;
 	fputs(ps_head, fout);
+	fputs(p ? psfgl : psdgl, fout);
+
+	// if a music font, give it a name
+	if (p) {
+		r = strrchr(p, '.');	// remove the file type
+		if (!r)
+			r = p + strlen(p) - 1;		// ')'
+		q = strrchr(p, DIRSEP);
+		if (!q)
+			q = strchr(p, '(');
+		q++;
+		fprintf(fout, "/music/%.*s def\n", (int) (r - q), q);
+	}
 
 	/* len su - up stem */
 	fprintf(fout, "/su{dlw x y M %.1f %.1f RM %.1f sub 0 exch RL stroke}!\n",
@@ -833,8 +894,7 @@ void define_symbols(void)
 		"	M dup 1 eq{\n"
 		"		pop\n"
 		"		0.6 -5.6 9.6 -9 5.6 -18.4 RC\n"
-		"		1.6 6 -1.3 11.6 -5.6 12.8 RC\n"
-		"		fill\n"
+		"		1.6 6 -1.3 11.6 -5.6 12.8 RC fill\n"
 		"	  }{\n"
 		"		1 sub{	currentpoint\n"
 		"			0.9 -3.7 9.1 -6.4 6 -12.4 RC\n"
@@ -842,8 +902,7 @@ void define_symbols(void)
 		"			fill 5.4 sub M\n"
 		"		}repeat\n"
 		"		1.2 -3.2 9.6 -5.7 5.6 -14.6 RC\n"
-		"		1.6 5.4 -1 10.2 -5.6 11.4 RC\n"
-		"		fill\n"
+		"		1.6 5.4 -1 10.2 -5.6 11.4 RC fill\n"
 		"	  }ifelse}!\n",
 		STEM_XOFF, STEM_YOFF, STEM_YOFF);
 
@@ -853,8 +912,7 @@ void define_symbols(void)
 		"	M dup 1 eq{\n"
 		"		pop\n"
 		"		0.6 5.6 9.6 9 5.6 18.4 RC\n"
-		"		1.6 -6 -1.3 -11.6 -5.6 -12.8 RC\n"
-		"		fill\n"
+		"		1.6 -6 -1.3 -11.6 -5.6 -12.8 RC fill\n"
 		"	  }{\n"
 		"		1 sub{	currentpoint\n"
 		"			0.9 3.7 9.1 6.4 6 12.4 RC\n"
@@ -862,8 +920,7 @@ void define_symbols(void)
 		"			fill 5.4 add M\n"
 		"		}repeat\n"
 		"		1.2 3.2 9.6 5.7 5.6 14.6 RC\n"
-		"		1.6 -5.4 -1 -10.2 -5.6 -11.4 RC\n"
-		"		fill\n"
+		"		1.6 -5.4 -1 -10.2 -5.6 -11.4 RC fill\n"
 		"	  }ifelse}!\n",
 		-STEM_XOFF, -STEM_YOFF, STEM_YOFF);
 
@@ -907,8 +964,7 @@ void define_symbols(void)
 		"	M dup 1 eq{\n"
 		"		pop\n"
 		"		0.6 -3.4 5.6 -3.8 3 -10 RC\n"
-		"		1.2 4.4 -1.4 7 -3 7 RC\n"
-		"		fill\n"
+		"		1.2 4.4 -1.4 7 -3 7 RC fill\n"
 		"	  }{\n"
 		"		{	currentpoint\n"
 		"			1 -3.2 5.6 -2.8 3.2 -8 RC\n"
@@ -924,8 +980,7 @@ void define_symbols(void)
 		"	M dup 1 eq{\n"
 		"		pop\n"
 		"		0.6 3.4 5.6 3.8 3 10 RC\n"
-		"		1.2 -4.4 -1.4 -7 -3 -7 RC\n"
-		"		fill\n"
+		"		1.2 -4.4 -1.4 -7 -3 -7 RC fill\n"
 		"	  }{\n"
 		"		{	currentpoint\n"
 		"			1 3.2 5.6 2.8 3.2 8 RC\n"
